/**
 * \file gre/lrtext.h
 * \brief GRE::LAYER_TEXT class definitions.
 *
 * \if NODOC
 * $Id: lrtext.h_v 1.24 2011/10/20 13:26:24 mju Exp $
 *
 * $Log: lrtext.h_v $
 * Revision 1.24  2011/10/20 13:26:24  mju
 * Remove redundant hdr.
 *
 * Revision 1.23  2007/03/30 22:35:40  dwilliss
 * Added virtual methods for datatips
 *
 * Revision 1.22  2007/03/28 20:04:17  dwilliss
 * added virtual methods
 *
 * Revision 1.21  2007/03/23 19:39:48  dwilliss
 * Added a virtual method
 *
 * Revision 1.20  2006/09/28 13:29:21  mju
 * Use elemstyl hdr.
 *
 * Revision 1.19  2006/09/15 15:31:06  mju
 * Serializer changes.
 *
 * Revision 1.18  2005/12/08 22:40:44  mju
 * Remove mem24 virtual
 *
 * Revision 1.17  2005/10/26 16:39:02  mju
 * Use gre namespace instead of legacy types.
 *
 * Revision 1.16  2005/10/14 17:32:11  mju
 * GRE namespace and treeview.
 *
 * Revision 1.15  2005/07/05 21:26:51  scowan
 * Changed some of the dispparm items.
 *
 * Revision 1.14  2005/04/21 20:26:37  mju
 * Remove unused flag.
 *
 * Revision 1.13  2005/03/31 16:57:06  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.12  2005/02/25 23:46:55  scowan
 * Added copy virtual method implementation.
 *
 * Revision 1.11  2005/02/14 21:39:49  mju
 * use device in usesTransparency.
 *
 * Revision 1.10  2005/01/20 16:57:47  mju
 * Redesign layer type registration.
 *
 * Revision 1.9  2004/10/15 21:29:30  ldyas
 * added needsmem24 method
 *
 * Revision 1.8  2004/10/15 14:32:47  ldyas
 * *** empty log message ***
 *
 * Revision 1.7  2004/10/14 22:05:38  ldyas
 * isantialiased is nonconst.
 *
 * Revision 1.6  2004/10/07 14:28:12  ldyas
 * added method for to allow IsAntialiased to detect smoothing in text style.
 *
 * Revision 1.5  2004/03/05 21:29:24  mju
 * Make dtor virtual if virtual fns.
 *
 * Revision 1.4  2004/02/18 23:37:30  scowan
 * Forward declare the text block from mgd namespace.
 *
 * Revision 1.3  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2003/07/30 15:29:08  mju
 * Ignore private sections.
 *
 * Revision 1.1  2002/09/20 17:47:44  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRTEXT_H
#define	INC_GRE_LRTEXT_H

#ifndef INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_RVC_TEXTOBJ_H
#include <rvc/textobj.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
namespace MGD {
	struct TEXTBLOCK;
	}
#endif

namespace GRE {
//===================================================================================================================

class LAYER_TEXT : public GRE::LAYER {
	public:

		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;

				FILEPATH m_filename;					//!< Filename of the object, this can be a simple text file
				MISTRING Text;
				MISTRING encoding;
				TEXTSTYLE Style;
				time_t LastModTime;					//!< Used to determine if linked file has changed
				char* String;							//!< For backwards compatibility
				RVC::TEXTOBJ m_TextObj;
				RVC::OBJECTDESC m_Desc;

				DISPPARM (LAYER_TEXT *layer);

				DISPPARM (const DISPPARM& rhs) { Copy(rhs); }

				virtual ~DISPPARM () { Free(); }

				DISPPARM& operator= (const DISPPARM& rhs) {
					if (this != &rhs) {
						Free();
						Copy(rhs);
						}
					return (*this);
					}

				//! Validate dispparm, adjusting if possible.
				virtual ERRVALUE Validate ();

			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				LAYER_TEXT *m_layer;
				void Copy (const DISPPARM&);
				void Free ();
				#endif //!< GENERATING_DOXYGEN_OUTPUT
			};

		//! Register this layer type
		static void RegisterType (
			);

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context							//!< SML context
			);

		//! Construct layer and insert into list
		explicit LAYER_TEXT (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None		//!< Creation flags
			);

		//! Destructor.
		virtual ~LAYER_TEXT ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class PRIV;
		PRIV* m_priv;

		void CleanupTextBlock ();
		ERRVALUE SetupTextBlock (MGD::TEXTBLOCK&);

		//! Overrides from GRE::LAYER.
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual int v_DataTipGetString (const DPOINT2D&, MISTRING&, SPATMOD::IMAGE::STAGE_ARRAY&, GRE::VIEW*);
		virtual bool v_DataTipHasAvailable () const;
		virtual bool v_DataTipHasEnabled () const;
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual bool v_IsAntialiased () const;
		virtual bool v_IsProportionVariable (DIMENSION) const;
		virtual const char* v_SerialGetTagName () const;
		virtual ERRVALUE v_SerialRead (SERIALIZER& serializer);
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual void v_SetRenderSize(GRE::LAYER::DIMENSION, double);
		virtual ERRVALUE v_TemplateRead (SERIALIZER& serializer);
		virtual int v_UpdateExtents (bool forceupdate);
		virtual bool v_UsesTransparency (const MGD::DEVICE *pDevice = 0) const;

		LAYER_TEXT (const LAYER_TEXT&);
		LAYER_TEXT& operator= (const LAYER_TEXT&);

		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

#define	TEXTDISPFLAG_Link				0x00000002	//!< Link to a file
#define	TEXTDISPFLAG_RVCObject		0x00000004	//!< Link is RVC object

//===================================================================================================================
}	// End namespace GRE


#endif	// INC_GRE_LRTEXT_H
