/**
 * \file gre/lrvector.h
 * \brief GRE::LAYER_VECTOR class definition.
 *
 * \if NODOC
 * $Id: lrvector.h_v 1.63 2013/03/08 21:25:55 mju Exp $
 *
 * $Log: lrvector.h_v $
 * Revision 1.63  2013/03/08 21:25:55  mju
 * Add dispparm thisclass typedef.
 *
 * Revision 1.62  2013/02/15 20:47:23  mju
 * Fix dox hdr.
 *
 * Revision 1.61  2012/06/13 14:55:37  mju
 * Remove v_appendPopupMenuItems.
 *
 * Revision 1.60  2012/06/05 14:32:48  mju
 * Add dispparm methods for access.
 *
 * Revision 1.59  2011/11/28 13:57:56  mju
 * Remove unused method.
 *
 * Revision 1.58  2011/10/20 13:09:55  mju
 * Remove redundant hdrs.
 *
 * Revision 1.57  2011/10/13 17:37:40  mju
 * Remove scalePointsZ to layer.
 *
 * Revision 1.56  2011/10/10 17:48:22  mju
 * Move callDrawFilter to layer.
 *
 * Revision 1.55  2010/10/13 19:53:06  mju
 * Add v_SnapPoint/IsAllowed.
 *
 * Revision 1.54  2010/08/19 17:40:49  mju
 * Use elemnum type.
 *
 * Revision 1.53  2010/04/23 16:54:59  mju
 * Use unsignedset.
 *
 * Revision 1.52  2008/05/15 17:55:19  mju
 * Chg v_constructLayerSEction.
 *
 * Revision 1.51  2008/05/12 17:00:06  mju
 * Add parm to constructLayerSection.
 *
 * Revision 1.50  2008/03/17 21:33:54  mju
 * Change v_getLayerView to v_getLayerSection.
 *
 * Revision 1.48  2007/03/28 17:59:27  mju
 * Change tempdrape stuff to be raster object pointers so can keep open.
 *
 * Revision 1.47  2007/03/12 17:43:02  mju
 * Add v_setObjectDescriptor.
 *
 * Revision 1.46  2006/10/16 17:54:05  mju
 * Remove unused drapeFileHandle member.
 *
 * Revision 1.45  2006/10/10 13:59:17  mju
 * Use miconstruct * instead of micon.
 *
 * Revision 1.44  2006/10/09 17:51:35  mju
 * Rmv v_setDftName.
 *
 * Revision 1.43  2006/10/02 17:48:25  mju
 * DataTip parms return as pointer.
 *
 * Revision 1.42  2006/09/28 15:55:35  mju
 * Use rvc objitem hdr.
 *
 * Revision 1.41  2006/09/27 16:17:53  mju
 * Use extrudedparms hdr and renamed classes.
 *
 * Revision 1.40  2006/09/25 20:52:32  mju
 * Add wireRead and drawWireframe.
 *
 * Revision 1.39  2006/09/15 17:57:40  mju
 * Remove labelplacement hdr.
 *
 * Revision 1.38  2006/09/15 15:31:14  mju
 * Serializer changes.
 * Add stuff for managing wireframe.
 *
 * Revision 1.37  2006/09/08 20:25:38  mju
 * Add wireframe member.
 *
 * Revision 1.36  2006/08/30 21:06:10  mju
 * Add dlgControlPanel member.
 *
 * Revision 1.35  2006/01/05 17:50:30  dwilliss
 * changed parm type on a method
 *
 * Revision 1.34  2005/12/08 22:40:49  mju
 * Remove mem24 virtual.
 *
 * Revision 1.33  2005/11/28 16:38:36  mju
 * Treeview update legend.
 *
 * Revision 1.32  2005/11/04 18:52:54  mju
 * enums in gre namespace.
 *
 * Revision 1.31  2005/11/03 18:27:18  mju
 * Mave v_menuAddToolItems protected instead of private.
 *
 * Revision 1.30  2005/10/27 13:44:54  mju
 * Fix header doc comments.
 *
 * Revision 1.29  2005/10/26 16:39:47  mju
 * Use gre namespace instead of legacy types.
 *
 * Revision 1.28  2005/10/19 17:21:29  mju
 * Put scriptcontext in gre namespace.
 *
 * Revision 1.27  2005/10/14 17:32:25  mju
 * GRE namespace and treeview.
 *
 * Revision 1.26  2005/09/12 18:32:59  vdronov
 * removed HasLabels
 *
 * Revision 1.25  2005/06/10 20:33:36  vdronov
 * added extra methods as IsManifold, ...
 *
 * Revision 1.24  2005/02/25 23:15:38  scowan
 * Added copy virtual method.
 *
 * Revision 1.23  2005/02/23 18:55:04  mju
 * Move label placement stuff to layerview.
 *
 * Revision 1.20  2005/01/27 16:47:37  vdronov
 * split v_Draw to Draw2D and Draw3D
 *
 * Revision 1.19  2005/01/20 16:58:01  mju
 * Redesign layer type registration.
 *
 * Revision 1.18  2004/10/15 21:28:13  ldyas
 * added needsmem24 method
 *
 * Revision 1.17  2004/10/01 17:20:25  mju
 * Add dlgparent to assignObject.
 *
 * Revision 1.15  2004/07/22 22:16:14  dwilliss
 * Moved LABLEPLACEMENT out of the layer.  It's now in a LAYERVIEW-derived
 * class which is private to mdvector.c
 *
 * Revision 1.14  2004/07/21 22:37:17  dwilliss
 * Added the construct layer view method
 *
 * Revision 1.12  2004/07/19 22:55:49  dwilliss
 * LABELPLACEMENT changes
 *
 * Revision 1.10  2004/04/08 17:20:33  vdronov
 * added m_VectorModifyStamp
 *
 * Revision 1.1  2002/09/19 17:09:20  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRVECTOR_H
#define	INC_GRE_LRVECTOR_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_GRE_LEGEND_H
#include <gre/legend.h>
#endif

#ifndef	INC_GRE_EXTRUDEDPARMS_H
#include <gre/extrudedparms.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class RASTER;
	}
#endif


namespace GRE {
//===================================================================================================================

//! GRE layer class for viewing Vector spatial object.
class LAYER_VECTOR : public LAYER {
	public:

		// LAYER_VECTOR::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:

				class DLG;										//!< Layer settings dialog, defined in <gre/uivector.h>

				RVC::OBJITEM m_ObjItem;
				RVC::OBJECTNAME m_GeorefName;				//!< Name of georeference object
				RVC::OBJITEM m_ObjItemStyle;
				POINTPARMS Point;
				LINEPARMS Line;
				POLYPARMS Poly;
				LABELPARMS Label;
				POINTPARMS Node;
				POINTSTYLE NodeDangleStyle;				//!< Special style for "dangling" nodes
				EXTRUDEDPOLYGONPARMS m_ExtrudedPolygon;
				EXTRUDEDPOINTPARMS m_ExtrudedPoint;
				bool EnablePolyFill3D;
				bool m_EnableLabelClipUnder;
				bool m_IsEditable;
				bool m_HasSaveInfo;
				bool m_ShowLabelBase;
				LINESTYLE m_LabelBaseStyle;
				mutable RVC::OBJITEM m_SaveObjItem;
				mutable RVC::OBJITEM m_SaveObjItemStyle;

				//! Default constructor.
				DISPPARM ();

				//! Copy constructor.
				DISPPARM (
					const DISPPARM& rhs
					): LAYER::DISPPARM(rhs) { Copy(rhs); }

				//! Destructor.
				virtual ~DISPPARM (
					) { Free(); }

				//! Assignment.
				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Free(); Copy(rhs); } return (*this); }

				void ClearGeorefName ()
					{ m_GeorefName.Clear(); }

				const RVC::OBJECTNAME& GetGeorefName () const
					{ return (m_GeorefName); }

				const RVC::OBJITEM& GetObjItem () const
					{ return (m_ObjItem); }

				const RVC::OBJITEM& GetObjItemStyle () const
					{ return (m_ObjItemStyle); }

				void SetGeorefName (
					const RVC::OBJECTNAME& GeorefName
					) { m_GeorefName = GeorefName; }

				//! Set to use specified object.
				ERRVALUE SetObject (
					const RVC::OBJITEM& objitem,		//!< Object to set
					bool keepsettings = false			//!< Retain current display settings
					);

				void SetSaveObjItems (
					const RVC::OBJITEM& ObjItemVector,
					const RVC::OBJITEM& ObjItemStyle
					) { m_SaveObjItem = ObjItemVector; m_SaveObjItemStyle = ObjItemStyle; m_HasSaveInfo = true; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef LAYER::DISPPARM BASECLASS;
				typedef LAYER_VECTOR::DISPPARM THISCLASS;

				static ERRVALUE SerializerCB_StyleFileName (SERIALIZER&, const SERIALIZERITEM *, void *, SERIALIZERITEM::ACTION);
				void Copy (const DISPPARM&);
				void Free ();
				void UpdateSaveInfo () const;

				//! Overrides from SERIALIZABLE.
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

				//! Overrides from LAYER::DISPPARM.
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();

				friend class LAYER_VECTOR;
				#endif //!< GENERATING_DOXYGEN_OUTPUT
			};	//! End of LAYER_VECTOR::DISPPARM

		//! Register this layer type
		static void RegisterType (
			);

		//! Method to install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context							//!< SML context
			);

		//! Construct layer and insert into list
		explicit LAYER_VECTOR (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		virtual ~LAYER_VECTOR ();

		//! Retrieve layer-specific display parameters
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	#ifdef USE_MGUI
		virtual void v_MenuAddLayerItems (MGUI::MENU& menu);
	#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		static void LegendMsgHandler (GRAPHICTREEITEM_MSG *msg);

		GRE::LEGEND_POINT m_PointLegend;
		GRE::LEGEND_LINE m_LineLegend;
		GRE::LEGEND_POLYGON m_PolyLegend;
		GRE::LEGEND_POINT m_NodeLegend;
		COMPINFO m_PointCompInfo;
		COMPINFO m_LineCompInfo;
		COMPINFO m_PolyCompInfo;
		COMPINFO m_NodeCompInfo;
		COMPINFO m_LabelCompInfo;
		GRE::SCRIPTCONTEXT_STYLE *m_pStyleSC;
		int m_StyleSegPoint;
		int m_StyleSegLine;
		int m_StyleSegPoly;
		int m_StyleSegNode;
		int m_StyleSegPoint3dBase;
		int m_StyleSegPoint3dExt;
		int m_StyleSegPoly3dExt;
		RVC::RASTER *m_pTempDrapeRast;
		RVC::RASTER *m_pTempDrapeMask;
	#ifdef USE_MGUI
		DISPPARM::DLG *m_pDlgControlPanel;
	#endif
		RVC::VECTOR *m_pObjVector;
		UINT32 m_VectorModifyStamp;
		GRE::GEOMETRIC3D *m_pGeometric3D;
		GRE::WIREFRAME *m_pWireframe;
	
		class PRIVDRAWDATA;
		friend class PRIVDRAWDATA;
		
		// temporary methods may go away ***************
		bool NewFeaturesFilter (LAYERDC& layerdc);
		bool IsModeWireFrame (LAYERDC& layerdc);
		bool IsSurfaceFlat (LAYERDC& layerdc);
		bool IsManifold (LAYERDC& layerdc);
		bool HasPointsOnStalks (LAYERDC& layerdc);
		bool HasExtrudedPolygons (LAYERDC& layerdc);
		// **********************************************

		ERRVALUE Draw2D (LAYERDC& layerdc);
		ERRVALUE Draw3D (LAYERDC& layerdc);
		int DrawPolyFilled3D (LAYERDC&);
		ERRVALUE DrawWireframe (LAYERDC& layerdc);
	#ifdef USE_MGUI
		void OnMenuExtractByRegion (MGUI::ID, bool);
		void OnMenuLegendElemListAddRmv (MGUI::ID, bool);
	#endif
		ERRVALUE OpenPrivateObject () const;
		ERRVALUE SetupDrape3D (LAYERDC&);
		ERRVALUE SetupLegend ();
		int SetupRegion ();
		ERRVALUE SetupScriptContext (GRE::SCRIPTCONTEXT_DRAW& sc);
		ERRVALUE WireReadExtrudedPoints (GRE::LAYERDC& layerdc);
		ERRVALUE WireReadExtrudedPolygons (GRE::LAYERDC& layerdc);
		ERRVALUE WireReadVector (GRE::LAYERDC& layerdc);

		//! Overrides from LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		virtual bool v_CanShowDetails () const;
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual const DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual void v_DataTipSetDefault (int component, bool force);
		virtual ERRVALUE v_DataTipSetParms (int component, const DATATIPPARMS& DataTipParms);
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual ERRVALUE v_DrawElementLow (LAYERDC& layerdc, ELEMTYPE ElemType, ELEMNUM ElemNum, const void *style);
		virtual ERRVALUE v_DrawElementsLow (LAYERDC& layerdc, ELEMTYPE ElemType, const UNSIGNEDSET *pElemSet, const void *style);
		virtual ERRVALUE v_DrawElementSample (const GRE::ELEMSAMPLE_DC& sampledc, const RVC::ELEMENT& element, const LRECT2D& samplerect);
		virtual ELEMNUM v_FindClosestElement (const DPOINT2D& point, double searchdist, double *distret, int component, const UNSIGNEDSET *pSearchSet, GRE::FINDFLAGS findflags);
		virtual int v_GetComponent (ELEMTYPE ElemType) const;
		virtual MICONSTRUCT* v_GetComponentIcon (int component) const;
		virtual COMPINFO* v_GetComponentInfo (int component);
		virtual MISTRING v_GetComponentName (int component) const;
		virtual const void* v_GetElemDefaultStyle (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawSelectionMode (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawStyleMode (ELEMTYPE ElemType) const;
		virtual ERRVALUE v_GetElemExtents (ELEMTYPE ElemType, ELEMNUM ElemNum, DRECT3D& Extents);
		virtual ERRVALUE v_GetElemSetByExtent (ELEMTYPE ElemType, const DRECT2D& Extent, UNSIGNEDSET& ElemSet);
		virtual ELEMTYPE v_GetElemType (int component) const;
		virtual bool v_GetGeorefName (RVC::OBJECTNAME& name, int component = 0) const;
		virtual bool v_GetLabelColor(LAYERDC& dc, const GRE::LABELPLACE& place, COLOR& color);
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElements (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual ERRVALUE v_GetStyleInfo (ELEMTYPE ElemType, RVC::OBJITEM& ObjItem, RVC::OBJECTNAME& TableName);
		virtual bool v_HasLegend () const;
		virtual bool v_IsTableUsableForComponent (int component, const RVC::DBTABLE& table) const;
	#ifdef USE_MGUI
		virtual void v_MenuAddRegionGenItems (MGUI::MENU& menu, ELEMTYPE ElemType);
	#endif
		virtual void v_OnNotifySurfaceChanged ();
	#ifdef USE_MGUI
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual ERRVALUE v_ReportRawElemInfo (MISTRING& infostr, ELEMTYPE ElemType, ELEMNUM ElemNum);
		virtual const char* v_SerialGetTagName () const;
		virtual ERRVALUE v_SerialRead (SERIALIZER& serializer);
		virtual void v_SetElemDrawSelectionMode (ELEMTYPE ElemType, UINT16 SelectMode);
		virtual void v_SetElemDrawStyleMode (ELEMTYPE ElemType, UINT16 StyleMode);
		virtual void v_SetLegendSampleFlags (MGD::STYLESAMPLEFLAGS SampleFlags);
		virtual void v_SetObjectDescriptor (int component, const RVC::DESCRIPTOR& descriptor);
		virtual ERRVALUE v_SetStyleObjItem (const RVC::OBJITEM& ObjItem);
		virtual int v_SnapPoint (DPOINT3D& ptObject, SNAPTO SnapTo, double MaxDistance, VIEW *view) const;
		virtual bool v_SnapPointIsAllowed (SNAPTO SnapTo) const;
		virtual int v_TestElementByRegion (ELEMTYPE ElemType, ELEMNUM ElemNum, const REGION2D& Region, TESTCRITERIA TestMode);
	#ifdef USE_MGUI
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
		virtual void v_TreeViewAddUpdateDetails (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual int v_UpdateExtents (bool forceupdate);
		virtual GRE::LAYERSECTION* v_ConstructLayerSection (GRE::VIEW* view);

		LAYER_VECTOR (const LAYER_VECTOR&);
		LAYER_VECTOR& operator= (const LAYER_VECTOR&);

		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRVECTOR_H
