/**
 * \file gre/uitreeview.h
 * \brief GRE::CTRL_TREEVIEW implementation.
 *
 * \if NODOC
 * $Id: uitreeview.h_v 1.20 2012/03/27 14:28:33 mju Exp $
 *
 * $Log: uitreeview.h_v $
 * Revision 1.20  2012/03/27 14:28:33  mju
 * Add stateidx_Blank.
 *
 * Revision 1.19  2011/12/27 16:57:40  mju
 * Use imagelst hdr.
 *
 * Revision 1.18  2011/11/28 17:22:35  mju
 * Fix dox dependencies.
 *
 * Revision 1.17  2011/10/20 13:33:58  mju
 * Remove redundant hdrs.
 *
 * Revision 1.16  2008/05/27 15:24:52  mju
 * Add parm to getLegendDC.
 *
 * Revision 1.15  2008/03/03 17:12:26  mju
 * Support drag-and-drop reordering.
 *
 * Revision 1.14  2008/02/29 17:09:04  mju
 * Add skiplocator parm to computeGroupStateIdx.
 *
 * Revision 1.13  2007/12/11 17:30:23  mju
 * Undo previous change - found better way.
 *
 * Revision 1.12  2007/12/11 17:23:09  mju
 * Add control for whether allowed to close top object.
 *
 * Revision 1.11  2007/11/28 18:53:49  mju
 * Add attach(viewable) method with parm to show top object.
 *
 * Revision 1.10  2007/08/20 14:23:38  mju
 * Remove unneeded qualifier.
 *
 * Revision 1.9  2007/08/02 13:40:54  mju
 * Add setAutoExpandWhenAddFirstChild.
 *
 * Revision 1.8  2006/08/01 13:53:30  mju
 * Add delegate to handle child visibility change for group.
 *
 * Revision 1.7  2006/02/16 19:00:13  mju
 * Change tooltip delegate.
 *
 * Revision 1.6  2005/11/10 18:47:15  mju
 * Add delegate for getItemInfo.
 *
 * Revision 1.5  2005/10/26 16:55:28  mju
 * Add GetItemFromObject.
 *
 * Revision 1.4  2005/10/24 17:57:46  mju
 * Add getViewable and imageidx for missing link table.
 *
 * Revision 1.3  2005/10/19 20:22:14  mju
 * Use gre namespace instead of legacy type names.
 *
 * Revision 1.2  2005/10/17 17:49:40  mju
 * Add setViewMode
 *
 * Revision 1.1  2005/10/14 17:33:39  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UITREEVIEW_H
#define	INC_GRE_UITREEVIEW_H

#ifndef	INC_GRE_ELEMSAMPLEDC_H
	#include <gre/elemsampledc.h>
#endif

#ifndef	INC_MGUI_TREE_H
	#include <mgui/tree.h>
#endif

#ifndef	INC_MGUI_IMAGELST_H
	#include <mgui/imagelst.h>
#endif

#ifndef	INC_MAP
	#include <map>
#endif

namespace GRE {
//===================================================================================================================

//! When using this class do not use the ID methods or values from MGUI::CTRL_TREE as the ID is managed
//! by CTRL_TREEVIEW and does not refer to any externally accessible object.
class CTRL_TREEVIEW : public MGUI::CTRL_TREE {
	public:

		enum IMAGEIDX {
			IMAGEIDX_Blank = 0,
			IMAGEIDX_Layout,
			IMAGEIDX_Group2D,
			IMAGEIDX_Group3D,
			IMAGEIDX_Legend,
			IMAGEIDX_Component_Node,
			IMAGEIDX_Component_Point,
			IMAGEIDX_Component_Line,
			IMAGEIDX_Component_Polygon,
			IMAGEIDX_Component_Edge,
			IMAGEIDX_Component_Triangle,
			IMAGEIDX_Component_Text,
			IMAGEIDX_Component_Element,
			IMAGEIDX_Table_Generic,
			IMAGEIDX_Table_Form,
			IMAGEIDX_Table_Linked,
			IMAGEIDX_Table_StdAttrib,
			IMAGEIDX_Table_Style,
			IMAGEIDX_Table_Internal,
			IMAGEIDX_Table_LinkedMissing,
			};

		//! Image index for 'state' indicators.
		enum STATEIDX {
			STATEIDX_None =							0,
			STATEIDX_Hidden =							1,
			STATEIDX_Visible =						2,
			STATEIDX_VisibleNotAll =				3,
			STATEIDX_ComponentNotSelectable =	4,
			STATEIDX_ComponentSelectable =		5,
			STATEIDX_Blank =							6,
			// Maximum allowed index is 15
			};

		enum VIEWMODE {
			VIEWMODE_Automatic =		0x00,
			VIEWMODE_Controls =		0x01,
			VIEWMODE_Legend =			0x02,
			VIEWMODE_Mixed =			0x03
			};

		class SHARED {
			public:
				//! Get reference to shared primary image list.
				MGUI::IMAGELIST& GetImageListPrimary (
					) { return (m_ImageListPrimary); }
				//! Get reference to shared state image list.
				MGUI::IMAGELIST& GetImageListState (
					) { return (m_ImageListState); }
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MGUI::IMAGELIST m_ImageListPrimary;
				MGUI::IMAGELIST m_ImageListState;
				SHARED ();
				SHARED (const SHARED&);
				SHARED& operator= (const SHARED&);
				friend class CTRL_TREEVIEW;
				#endif
			};

		//! Delegate for pointer button.
		typedef FastDelegate<bool(CTRL_TREEVIEW&,ITEMHND,HITRESULT,const LPOINT2D&,MGUI::POINTERBUTTON,MGUI::KEYSTATE)> DELEGATE_BUTTON;

		//! Delegate for custom drawing.
		typedef FastDelegate<MGUI::CUSTOMDRAW::RETVALUE(CTRL_TREEVIEW&,MGUI::CUSTOMDRAW_TREE&)> DELEGATE_CUSTOMDRAW;

		//! Delegate for item.
		typedef FastDelegate<void(CTRL_TREEVIEW&,ITEMHND)> DELEGATE_ITEM;

		//! Delegate for obtaining owner-supplied item info.
		typedef FastDelegate<void(CTRL_TREEVIEW&,ITEMVALUES,ITEMINFO&)> DELEGATE_GETITEMINFO;

		//! Delegate for child visibility changed.
		typedef FastDelegate<void(CTRL_TREEVIEW&,ITEMHND,CHILDVISIBILITY)> DELEGATE_VISIBILITYCHANGED;

		//! Delegate for child visibility changing.
		typedef FastDelegate<bool(CTRL_TREEVIEW&,ITEMHND,CHILDVISIBILITY)> DELEGATE_VISIBILITYCHANGING;

		//! Delegate for tooltip request.
		typedef FastDelegate<bool(CTRL_TREEVIEW&,ITEMHND,HITRESULT,const LPOINT2D&,MGUI::CTRL_TOOLTIP&)> DELEGATE_TOOLTIP;

		//! Get shared information for use by all object trees.
		static SHARED& GetShared (
			);

		//! Default constructor.
		CTRL_TREEVIEW ();

		//! Destructor.
		virtual ~CTRL_TREEVIEW ();

		//! Attach to VIEWABLE.
		void Attach (
			VIEWABLE *viewable				//!< Viewable object to attach
			);

		//! Attach to VIEWABLE.
		void Attach (
			VIEWABLE *viewable,				//!< Viewable object to attach
			bool ShowTopObject				//!< Whether to show topmost object (viewable) in tree
			);

		//! Attach to GRE_VIEW and associated viewable object.
		void Attach (
			GRE::VIEW *view						//!< View to attach
			);

		//! Attach to all top GRE objects.
		void AttachAll (
			);

		//! Detach from viewable/view.
		void Detach (
			);

		//! Find handle for first item associated with specified pointer.
		ITEMHND GetItemFromPtr (
			const void *ptr
			) const;

		//! Find handle for first item associated with specified pointer.
		ITEMHND GetItemFromObject (
			const GRE::OBJECT *object
			) const;

		//! Get element sample drawing context for legend.
		const ELEMSAMPLE_DC& GetLegendDC (
			GRE::LEGEND& legend,						//!< Legend
			MGD::CONTEXT *pgc
			);

		//! Get pointer previously set for handle.
		void* GetPtr (
			ITEMHND hndItem
			) const;

		//! Get view associated with control, if any.
		GRE::VIEW* GetView (
			) const { return (m_view); }

		//! Get viewable associated with control, if any.
		GRE::VIEWABLE* GetViewable (
			) const { return (m_viewable); }

		//! Get current view mode.
		VIEWMODE GetViewMode (
			) const { return (m_viewmode); }

		//! Insert item into tree.
		ITEMHND Insert (
			const MISTRING& name,
			ITEMHND hndParent,
			ITEMHND hndInsertAfter,
			int ImageIdx,
			void *object,
			STATEIDX StateIdx = STATEIDX_None
			);

		bool IsAllBold (
			ITEMHND hndItem
			) const;

		//! Set whether to auto-expand layout/group when add first child.
		//! Default is to auto-expand.
		void SetAutoExpandWhenAddFirstChild (
			bool AutoExpand
			) { m_AutoExpandWhenAddFirstChild = AutoExpand; }

		//! Set delegate to call when child visibility changed.
		void SetDelegateOnChildVisibilityChanged (
			ITEMHND hndItem,
			DELEGATE_VISIBILITYCHANGED delegate
			);

		//! Set delegate to call when child visibility about to change.
		void SetDelegateOnChildVisibilityChanging (
			ITEMHND hndItem,
			DELEGATE_VISIBILITYCHANGING delegate
			);

		//! Set delegate to call for custom drawing.
		void SetDelegateOnCustomDraw (
			ITEMHND hndItem,
			DELEGATE_CUSTOMDRAW delegate
			);

		//! Set delegate to call to get item info.
		void SetDelegateOnGetItemInfo (
			ITEMHND hndItem,
			DELEGATE_GETITEMINFO delegate
			);

		//! Set delegate to call when item position changed.
		void SetDelegateOnItemChangePosition (
			ITEMHND hndItem,
			DELEGATE_ITEM delegate
			);

		//! Set delegate to call when pointer button pressed.
		//! Delegate must return true if handled, false to perform default action.
		void SetDelegateOnPointerButtonDown (
			ITEMHND hndItem,
			DELEGATE_BUTTON delegate
			);

		//! Set delegate to call when tooltip requested.
		void SetDelegateOnToolTipRequest (
			ITEMHND hndItem,
			DELEGATE_TOOLTIP delegate
			);

		//! Set pointer to associate with item.
		void SetPtr (
			ITEMHND hndItem,
			void *ptr
			);

		void SetViewMode (
			VIEWMODE viewmode
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class ITEM;
		typedef std::map<void*,ITEMHND> PTRHNDMAP;

		VIEWMODE m_viewmode;
		VIEWABLE *m_viewable;
		GRE::VIEW *m_view;
		bool m_IncludeTopObject;
		bool m_AutoExpandWhenAddFirstChild;
		PTRHNDMAP m_PtrHndMap;
		ELEMSAMPLE_DC m_LegendDC;

		static void GRECB_Object (
			const GRE::MSG *msg,
			void *vpTree
			);

		void Attach (VIEWABLE *viewable, GRE::VIEW *view, bool ShowTopObject);
		unsigned int ComputeGroupStateIdx (GRE::GROUP *group, bool skiplocator = false) const;

		ITEM* GetItem (
			ITEMHND hndItem
			) const { return (static_cast<ITEM*>(reinterpret_cast<void*>(GetItemID(hndItem)))); }

		ITEM* GetItem (ITEMHND hndItem, bool create);
		ITEMHND InsertGroupItem (GRE::GROUP *group, ITEMHND hndParent, ITEMHND hndInsertAfter);
		ITEMHND InsertGroupSubItems (GRE::GROUP *group, ITEMHND hndParent, ITEMHND hndInsertAfter);
		ITEMHND InsertLayoutItem (GRE::LAYOUT *layout, ITEMHND hndParent, ITEMHND hndInsertAfter);
		void InsertLayoutSubItems (GRE::LAYOUT *layout, ITEMHND hndParent);

		void OnGreMsg (const GRE::MSG& msg);
		void OnGroupChangePosition (CTRL_TREEVIEW&, ITEMHND);
		void OnGroupChildVisibilityChanged (CTRL_TREEVIEW&, ITEMHND, CHILDVISIBILITY);
		bool OnGroupPointerButton (CTRL_TREEVIEW&, ITEMHND, HITRESULT, const LPOINT2D&, MGUI::POINTERBUTTON, MGUI::KEYSTATE);
		bool OnGroupToolTipRequest (CTRL_TREEVIEW&, ITEMHND, HITRESULT, const LPOINT2D&, MGUI::CTRL_TOOLTIP&);
		bool OnLayoutPointerButton (CTRL_TREEVIEW&, ITEMHND, HITRESULT, const LPOINT2D&, MGUI::POINTERBUTTON, MGUI::KEYSTATE);
		bool OnLayoutToolTipRequest (CTRL_TREEVIEW&, ITEMHND, HITRESULT, const LPOINT2D&, MGUI::CTRL_TOOLTIP&);
		void PopupMenuGroup(GRE::GROUP*, const LPOINT2D&);
		void PopupMenuLayout(GRE::LAYOUT*, const LPOINT2D&);
		void RemoveGreMsgHandler ();
		void SetAllStateImages ();
		void SetAllStateImages (GRE::LAYOUT*);
		void SetAllStateImages (GRE::GROUP*);
		void SetGroupStateImage (GRE::GROUP*, ITEMHND hndGroupItem = ITEMHND_None);
		bool UpdateGroupLegend (GRE::GROUP*, bool forceupdate = false);

		// MGUI::CTRL_TREE overrides.
		virtual void v_OnChildVisibilityChanged (ITEMHND hndItem, CHILDVISIBILITY visibility);
		virtual bool v_OnChildVisibilityChanging (ITEMHND hndItem, CHILDVISIBILITY visibility);
		virtual MGUI::CUSTOMDRAW::RETVALUE v_OnCustomDraw (MGUI::CUSTOMDRAW_TREE& CustomDraw);
		virtual void v_OnDestroyItem (ITEMHND hndItem);
		virtual void v_OnGetItemInfo (ITEMVALUES RequestedValue, ITEMINFO& ItemInfo);
		virtual void v_OnItemChangePosition (ITEMHND hndItem);
		virtual void v_OnPointerButtonDown (ITEMHND hndItem, HITRESULT HitResult, const LPOINT2D& point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		virtual void v_OnPreInsertItem (ITEMINFO& ItemInfo);
		virtual bool v_OnToolTipRequest (ITEMHND hndItem, HITRESULT HitResult, const LPOINT2D& point, MGUI::CTRL_TOOLTIP& ToolTipCtrl);

		CTRL_TREEVIEW (const CTRL_TREEVIEW&);
		CTRL_TREEVIEW& operator= (const CTRL_TREEVIEW&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_UITREEVIEW_H
