 /**
 *	\file mi32/arraycache.h
 *	\brief Definitions of ARRAYCACHE template class.
 *
 * \if NODOC
 * $Id: arraycache.h_v 1.4 2012/09/18 13:46:01 mju Exp $
 *
 * $Log: arraycache.h_v $
 * Revision 1.4  2012/09/18 13:46:01  mju
 * Attempt dox fix.
 *
 * Revision 1.3  2011/10/24 12:38:23  mju
 * Fix typo.
 *
 * Revision 1.2  2011/10/21 20:39:47  mju
 * Remove unused member, hdr, and exclude private section from dox.
 *
 * Revision 1.1  2008/05/30 22:23:39  vdronov
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_ARRAYCACHE_H
#define	INC_MI32_ARRAYCACHE_H

#ifndef INC_RVC_ARRAY_H
#include <rvc/array.h>
#endif

#ifndef INC_MI32_MICACHE_H
#include <mi32/micache.h>
#endif


template <typename _TT> class ARRAYCACHE : public MICACHEIO {
	public:

		ARRAYCACHE (
			) : m_pObjArray(0)
			{ }

		virtual ~ARRAYCACHE () {}

		void Clear (
			) {
			m_Cache.Flush();
			m_Cache.Free();
			m_pObjArray = 0;
			}

		ERRVALUE Init (
			const INT32 NumCache, 
			RVC::ARRAY *pObjArray
			) {
			ERRVALUE err = m_Cache.Create(NumCache, sizeof(_TT), *this);
			if (err < 0) return err;
			m_pObjArray = pObjArray;
			return 0;
			}

		ERRVALUE Read (
			const INT32 itemid,
			_TT& item
			) { return (m_Cache.Read(itemid, 0, sizeof(_TT), &item, m_pObjArray)); }

		ERRVALUE Select (
			const INT32 itemid,
			MICACHE::ITEMINFO& iteminfo
			) { return (m_Cache.Select(itemid, iteminfo, MICACHE::SELECTFLAG_None, m_pObjArray)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::ARRAY *m_pObjArray;
		MICACHE m_Cache;

		ERRVALUE CacheLoad (
			MICACHE::ITEMINFO &iteminfo
			) { 
			RVC::ARRAY* Array = (RVC::ARRAY*)iteminfo.GetUserDataPtr();
			return Array->Read(iteminfo.GetItemID(), iteminfo.GetMemPtr()); 
			}

		ERRVALUE CacheSave (
			MICACHE::ITEMINFO& iteminfo
			) { 
			RVC::ARRAY* Array = (RVC::ARRAY*)iteminfo.GetUserDataPtr();
			return Array->Write(iteminfo.GetItemID(), iteminfo.GetMemPtr()); 
			}

		#endif
	};

#endif
