/**
 * \file colorarray.h <mi32/colorarray.h>
 * \brief COLOR_ARRAY class definition.
 *
 * \if NODOC
 *
 * $Id: colorarray.h_v 1.4 2007/01/19 22:44:04 mju Exp $
 *
 * $Log: colorarray.h_v $
 * Revision 1.4  2007/01/19 22:44:04  mju
 * Export from dll and add findClosestColor fn.
 *
 * Revision 1.3  2007/01/19 16:09:08  mju
 * Add commented-out code for future use.
 *
 * Revision 1.2  2007/01/18 14:14:33  mju
 * Add ctor/op= from base.
 *
 * Revision 1.1  2005/12/15 18:07:17  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_COLORARRAY_H
#define	INC_MI32_COLORARRAY_H

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//! Container for array (palette) of colors.
class CLASSLIBEXPORT COLOR_ARRAY : public SIMPLE_ARRAY<COLOR> {
	public:
		//! Find closest color in array to specified color.
		//! Distance is computed in RGB colorspace.
		//! Performs linear search of array, so may not be efficient for multiple calls.
		//! @return Index of closest color, -1 if array is empty.
		INT32 FindClosestColor (
			const COLOR& color
			) const;
	};

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_COLORARRAY_H
