/**
 * \file mi32/mucdefns.h
 * \brief Definitions for Muc functions
 *
 * \if NODOC
 * $Id: mucdefns.h_v 1.1 2008/11/14 15:30:04 mju Exp $
 *
 * $Log: mucdefns.h_v $
 * Revision 1.1  2008/11/14 15:30:04  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MUCDEFNS_H
#define	INC_MI32_MUCDEFNS_H

#ifndef INC_MI32_CHARENCODING_H
#include <mi32/charencoding.h>
#endif

#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif

#define MUCVERSION	2

#ifndef GENERATING_DOXYGEN_OUTPUT
/* Structure of the header of unicode2.ref.  All numbers are stored
 * in LoHi byte order in unicode2.ref.  HiLo machines will have to
 * swap
 */
struct MUCFILEHDR {
	UINT32 FileHdrSize;		//!<  Size of this structure 
	UINT32 EncHdrSize;			//!<  sizeof(MUCENCODEHDR) 
	UINT32 Version;				//!<  MUCVERSION 
	UINT32 NumEncodings;		//!<  Number of MUCENCODEHDRs that follow 
	};

struct MUCENCODEHDR {
	MIUNICODE name[16];	
	INT32 offset;				//!<  Offset into file of data for this encoding 
	INT32 numpairs;
	};

struct UCDATA;

#endif // GENERATING_DOXYGEN_OUTPUT

/*
**	Constants for MucCharsetToUnicode()/MucUnicodeToCharset()
*/
#define MASK_ALG                 0x7FFF0000
#define MASK_CHARSET             0x0000FFFF

#define CHARALG_SJIS             0x00010000
#define CHARALG_EUC             	0x00020000
#define CHARALG_GL2GR				0x00040000	//!< Set high bits
#define CHARALG_GR2GL				0x00080000	//!< Strip high bits

#define CHARSET_Unicode          0x00000000
#define CHARSET_GB_2312          0x00000001
#define CHARSET_GB_12345         0x00000002
#define CHARSET_GB_7589          0x00000003
#define CHARSET_GB_7590          0x00000004
#define CHARSET_GB_Han           0x00000005
#define CHARSET_GB_8565          0x00000006
#define CHARSET_BigFive          0x00000007
#define CHARSET_CNS_11543_01     0x00000008
#define CHARSET_CNS_11543_02     0x00000009
#define CHARSET_CNS_11543_14     0x0000000A
#define CHARSET_JIS_X_0208_1990  0x0000000B
#define CHARSET_JIS_X_0212_1990  0x0000000C
#define CHARSET_KS_C_5601_1987   0x0000000D	//!< Wansung
#define CHARSET_KS_C_5657_1991   0x0000000E
#define CHARSET_ISO_8859_1      	0x0000000F      //!<  ASCII 
#define CHARSET_ISO_8859_2      	0x00000010     
#define CHARSET_ISO_8859_3      	0x00000011    
#define CHARSET_ISO_8859_4      	0x00000012  
#define CHARSET_ISO_8859_5      	0x00000013
#define CHARSET_ISO_8859_6      	0x00000014
#define CHARSET_ISO_8859_7      	0x00000015
#define CHARSET_ISO_8859_8      	0x00000016
#define CHARSET_ISO_8859_9      	0x00000017
#define CHARSET_JIS_ROMAN			0x00000018
#define CHARSET_GB_ROMAN			0x00000019	//!<  From ISO 2022 
#define CHARSET_JIS_C_6226			0x0000001A	//!<  From ISO 2022 
#define CHARSET_HW_KATAKANA		0x0000001B	//!<  From ISO 2022 
#define CHARSET_WinCP_Turk			0x0000001C	//!<  Windows Codepage (cp1254)
#define CHARSET_WinCP_EE			0x0000001D	//!<  Windows Codepage (cp1250)
#define CHARSET_WinCP_Cyrl			0x0000001E	//!<  Windows Codepage (cp1251)
#define CHARSET_WinCP_Greek		0x0000001F	//!<  Windows Codepage (cp1253)
#define CHARSET_WinCP_Arab			0x00000020	//!<  Windows Codepage (cp1256)
#define CHARSET_WinCP_Hebr			0x00000021	//!<  Windows Codepage (cp1255)
#define CHARSET_DOSCP_437			0x00000022	//!<  DOS Codepage 
#define CHARSET_DOSCP_850			0x00000023	//!<  DOS Codepage 
#define CHARSET_DOSCP_852			0x00000024	//!<  DOS Codepage 
#define CHARSET_DOSCP_857			0x00000025	//!<  DOS Codepage 
#define CHARSET_DOSCP_861			0x00000026	//!<  DOS Codepage 
#define CHARSET_DOSCP_863			0x00000027	//!<  DOS Codepage 
#define CHARSET_DOSCP_865			0x00000028	//!<  DOS Codepage 
#define CHARSET_IBMCP_869			0x00000029	//!<  IBM Codepage 
#define CHARSET_IBMCP_855			0x0000002A	//!<  IBM Codepage 
#define CHARSET_IBMCP_864			0x0000002B	//!<  IBM Codepage 
#define CHARSET_IBMCP_1040			0x0000002C	//!<  IBM Codepage 
#define CHARSET_IBMCP_1041			0x0000002D	//!<  IBM Codepage 
#define CHARSET_IBMCP_1043			0x0000002E	//!<  IBM Codepage 
#define CHARSET_TIS620_2529		0x0000002F	//!<  Thai (BDF font order) 
#define CHARSET_WinCP_Thai			0x00000030	//!<  Thai Windows ordering 
#define CHARSET_MI_Thai			   0x00000031	//!<  Thai Windows ordering 
#define CHARSET_WinCP_ANSI		   0x00000032	//!<  Windows ANSI Code Page 
#define CHARSET_ArabicUC		   0x00000033	//!<  The Arabic range of Unicode 
#define CHARSET_MacArabic			0x00000034
#define CHARSET_MacCroatian		0x00000035
#define CHARSET_MacCyrillic		0x00000036
#define CHARSET_MacDingbats		0x00000037
#define CHARSET_MacCentEurope		0x00000038
#define CHARSET_MacGreek			0x00000039
#define CHARSET_MacHebrew			0x0000003A
#define CHARSET_MacIcelandic	 	0x0000003B
#define CHARSET_MacJapanese		0x0000003C
#define CHARSET_MacRomanian		0x0000003D
#define CHARSET_MacRoman			0x0000003E
#define CHARSET_MacSymbol			0x0000003F
#define CHARSET_MacThai				0x00000040
#define CHARSET_MacTurkish			0x00000041
#define CHARSET_MacUkrainian		0x00000042
#define CHARSET_WinCP_Baltic		0x00000043
#define CHARSET_IBM_1046			0x00000044	//!<  IBM Arabic encoding 
#define CHARSET_KS_C_5601_Unif	0x00000045	//!< Unified Hangul (UHang)
#define CHARSET_KS_C_5601_1992	0x00000046	//!< Johab
#define CHARSET_ISCII_Devanagari	0x00000047	//!< Indic
#define CHARSET_ISCII_Bengali		0x00000048	//!< Indic
#define CHARSET_ISCII_Gurmukhi	0x00000049	//!< Indic
#define CHARSET_ISCII_Gujarati	0x0000004A	//!< Indic
#define CHARSET_ISCII_Oriya		0x0000004B	//!< Indic
#define CHARSET_ISCII_Tamil		0x0000004C	//!< Indic
#define CHARSET_ISCII_Telugu		0x0000004D	//!< Indic
#define CHARSET_ISCII_Kannada		0x0000004E	//!< Indic
#define CHARSET_ISCII_Malayalam	0x0000004F	//!< Indic
#define CHARSET_KOI8					0x00000050	//!< Russian/Ukranian/Etc.
#define CHARSET_CP932				0x00000051	//!< Shift-JIS with MS Extensions
#define CHARSET_MAX              0x00000051

/* Some aliases... */
#define CHARSET_ISO_Latin_1      CHARSET_ISO_8859_1	
#define CHARSET_ASCII            CHARSET_ISO_Latin_1
#define CHARSET_JIS_0208			CHARSET_JIS_X_0208_1990
#define CHARSET_JIS_0212			CHARSET_JIS_X_0212_1990
#define CHARSET_SHIFT_JIS        (CHARALG_SJIS | CHARSET_JIS_X_0208_1990)

#define UCCONV_NoByteOrderMark	0x00000001

#ifndef __cplusplus
typedef
#endif
//! Script tags
//! These are used mostly for when multiple "scripts" (as in writing systems)
//! use the same character set but have different rules.  A good example of 
//! this is the Indic languages which mostly share the ISCII character set
//! but Devanagari and Bengali have different formatting rules.
//! The numbers below are not random gibberish as it may seem.  If you take
//! each pair of hex digits, they form an ASCII value so that together they
//! make up a 4 letter abbreviation.  This is how these tags are encoded in
//! TrueType fonts.
enum SCRIPTTAG {

	 SCRIPTTAG_arab = 0x61726162, //!< 'arab' *** Arabic
	 SCRIPTTAG_armn = 0x61726D6E, //!< 'armn' *** Armenian: no MS definition ***
	 SCRIPTTAG_beng = 0x62656E67, //!< 'beng' *** Bengali (Indic)
	 SCRIPTTAG_bpmf = 0x62706D66, //!< 'bpmf' *** Bopomofo: no MS definition ***
	 SCRIPTTAG_cyrl = 0x6379726C, //!< 'cyrl' *** Cyrillic
	 SCRIPTTAG_deva = 0x64657661, //!< 'deva' *** Devanagari (Indic)
	 SCRIPTTAG_grek = 0x6772656B, //!< 'grek' *** Greek
	 SCRIPTTAG_grgn = 0x6772676E, //!< 'grgn' *** Georgian: no MS definition ***
	 SCRIPTTAG_gujr = 0x67756A72, //!< 'gujr' *** Gujarati (Indic)
	 SCRIPTTAG_hang = 0x68616E67, //!< 'hang' *** 
	 SCRIPTTAG_hani = 0x68616E69, //!< 'hani' *** 
	 SCRIPTTAG_hebr = 0x68656272, //!< 'hebr' *** Hebrew
	 SCRIPTTAG_kana = 0x6B616E61, //!< 'kana' *** 
	 SCRIPTTAG_knbn = 0x6B6E626E, //!< 'knbn' *** Kanbun: no MS definition ***
	 SCRIPTTAG_knda = 0x6B6E6461, //!< 'knda' *** Kannada (Indic)
	 SCRIPTTAG_laoS = 0x6C616F20, //!< 'lao ' *** Lao: no MS definition ***
	 SCRIPTTAG_latn = 0x6C61746E, //!< 'latn' *** Latin
	 SCRIPTTAG_mlym = 0x6D6C796D, //!< 'mlym' *** Malayalam (Indic)
	 SCRIPTTAG_orya = 0x6F727961, //!< 'orya' *** Oriya (Indic)
	 SCRIPTTAG_punj = 0x70756E6A, //!< 'punj' *** punjabi == gurmukhi
	 SCRIPTTAG_taml = 0x74616D6C, //!< 'taml' *** Tamil (Indic)
	 SCRIPTTAG_telu = 0x74656C75, //!< 'telu' *** Telugu (Indic)
	 SCRIPTTAG_thai = 0x74686169, //!< 'thai' *** Thai
	 SCRIPTTAG_tibt = 0x74696174, //!< 'tibt' *** Tibetan
	 	
	 SCRIPTTAG_neut = 0x4E455554, //!< 'NEUT'
	 SCRIPTTAG_puse = 0x50555345, //!< 'PUSE'
	 SCRIPTTAG_spcl = 0x5350434C, //!< 'SPCL'
	 SCRIPTTAG_surr = 0x53555252, //!< 'SURR'

	 SCRIPTTAG_Default = 0x00000000 //!< ''
	}
#ifndef __cplusplus
	SCRIPTTAG
#endif
	;


//!
//!	Some special glyph flags.  The method we use to generate GLYPHCONTEXTBITs
//!	requires that PartOfRtoLWord be 1 and that we don't use bits 0x02 and 0x04
//! here.  The PartOfRtoLWord flag will be shifted and OR'd with the bits of 
//! the glyphs on either side of it to create GLYPHCONTEXT bits.  
//! Yes, they're bytes, not longs.  Don't muck with `em!
//!
#define MucGLYPHFLAG_PartOfRtoLWord		0x01
#define MucGLYPHFLAG_NonJoining			0x80
#define MucGLYPHFLAG_Mark					0x40	

#define MucGLYPHDIRECTION_LtoR			0
#define MucGLYPHDIRECTION_RtoL			1
#define MucGLYPHDIRECTION_Weak			2

#if defined(__cplusplus)
extern "C" {
#endif

//! Decompose a Unicode string into Canonical form.
//!
//! In a cononical Unicode string, all decomposable characters are decomposed.
//! For example, 0x00E9 (LATIN SMALL LETTER E WITH ACUTE) is represented as
//! 0x0065 (LATIN SMALL LETTER E) + 0x0301 (COMBINING ACUTE ACCENT)
//!
//! The caller should free the returned string
LIBEXPORT MIUNICODE* MucDecompose (
	const MIUNICODE* string
	);

//! Recompose a Canonical form Unicode string into the combined forms
//!
//! In a cononical Unicode string, all decomposable characters are decomposed.
//! For example, 0x00E9 (LATIN SMALL LETTER E WITH ACUTE) is represented as
//! 0x0065 (LATIN SMALL LETTER E) + 0x0301 (COMBINING ACUTE ACCENT).  This
//! function puts them back together.
//!
//! The caller should free the returned string
LIBEXPORT MIUNICODE* MucRecompose (
	const MIUNICODE* string
	);

//! Initialize a MIUNICODE conversion handle.
LIBEXPORT int MucInitConversion (
	void** handle, 
	const char* encoding, 
	UINT32 flags
	);

//! Stop a MIUNICODE conversion handle.
LIBEXPORT void MucStopConversion (
	void* handle
	);

//! Convert a single character from a given character set to MIUNICODE.
LIBEXPORT int MucCharsetToUnicode (
	int val, 
	int charset
	);

//! Convert a string characters from a given character set to MIUNICODE.
//! Operates in-place.
LIBEXPORT int MucCharsetToUnicodeBuf (
	MIUNICODE *buf, 
	int charset
	);	

//! Convert string from anything to MIUNICODE.
//!
//! The "vcd" must have been initialized by calling MucInitConversion().
//! Assumes vinbuf is whatever vcd was initialized to.
//! Caller must free outbuf when done
LIBEXPORT int MucConvertToUnicode (
	void *vcd, 
	const void *vinbuf, 
	MIUNICODE **outbuf
	);

//! Convert string from anything to ISO-2022.
//!
//! The "vcd" must have been initialized by calling MucInitConversion().
//! Assumes vinbuf is whatever vcd was initialized to.
//! Caller must free outbuf when done
LIBEXPORT int MucConvertToISO2022 (
	void *vcd, 
	const void *vinbuf, 
	char **outbuf
	);

//! Convert string from ISO-2022 to anything.
LIBEXPORT int MucConvertFromISO2022 (
	void *vcd, 
	const void *vinbuf, 
	char **outbuf
	);

//! Convert string from MIUNICODE to anything.
LIBEXPORT int MucConvertFromUnicode (
	void *vcd, 
	const MIUNICODE *ucbuf, 
	void **outbufp
	);

//! Convert string from MIUNICODE by calling callbacks.
//!
//! This function can be used to convert a string from MIUNICODE to
//! just about anything.  It assumes ISO-8859-1 as an initial
//! character set. 
//!
//! NewCharsetFunc() 
//! 	will be called when the function detects a character not available
//! 	in the current character set.  It tells you what character set it
//! 	was in and what character set it wants to switch to.  At this point,
//! 	you can do whatever you need to do to switch fonts to one that 
//! 	supports the requested character set.  You should return 0 or an error
//! 	code < 0.
//!
//! AddCharsFunc() is called when an internal buffer fills up or just before 
//! 	calling NewCharsetFunc() to let add the characters to your output 
//! 	buffer or display them or whatever it is you're doing.  The "characters" 
//! 	passed to you may be multi-byte.  If this is the case, "len" is in 
//! 	bytes.  
//!
LIBEXPORT int MucConvertUnicodeGen (
	const MIUNICODE* inbuf, 
	int(*NewCharsetFunc)(int NewCharset, int OldCharset, void* UserData), 
	int(*AddCharsFunc)(UINT8* chars, int len, void* UserData), 
	void* UserData, 
	UINT32 flags
	);


//! Convert a MIUNICODE string to ISO-2022.
//!
//! The caller should free the buffer returned
LIBEXPORT int MucConvertUnicodeToISO2022 (
	const MIUNICODE *inbuf, 
	char **outbuf
	);

//! Convert an ISO-2022 string to MIUNICODE.
//!
//! The caller should free the buffer returned
LIBEXPORT int MucConvertISO2022ToUnicode (
	const void*, 
	MIUNICODE**
	);

//! Convert an ISO-2022a string to MIUNICODE.
//!
//! ISO-2022a is a MicroImages invention -- a slightly modified version
//! of ISO-2022 in which all backslashes are doubled.  This is done because
//! X resource files try to intrepret back slashes as something special
//! even it the backslash is the 2nd byte of a 2-byte character.
//!
//! The caller should free the buffer returned
LIBEXPORT int MucConvertISO2022aToUnicode (
	const void*, 
	MIUNICODE**
	);

//! Convert UTF8 to MIUNICODE (UTF-16).
//!
//! Warning!  This function assumes that the pointer passed in
//! for outstr is either NULL or can be realloc'd as needed.
//! You must therefore initialize it to somthing.
LIBEXPORT int MucConvertUTF8ToUnicode (
	const UINT8* instr, 
	MIUNICODE** outstr
	);

//! Convert MIUNICODE (UTF-16) to UTF8.
//!
//! Warning!  This function assumes that the pointer passed in
//! for outstr is either NULL or can be realloc'd as needed.
//! You must therefore initialize it to somthing.
LIBEXPORT int MucConvertUnicodeToUTF8 (
	const MIUNICODE* instr, 
	UINT8** outstr
	);

#ifdef __cplusplus
//! Perform glyph substitutions on a MIUNICODE string.
//!
//! This function returns a copy of the given string after performing
//! glyph substitutions based on the decomposition data in 
//! UnicodeData.txt (which we've cooked into a compact form, ucdata.ref)
//! 
//! Note: This function assumes the input string is in logical order and
//! does not flip it in any way.
//!
//! @param instr
//!	String to convert
//! @param CB_AllowSub
//! 	Aallows you to intercept and deny substitutions.
//! 	A good use for this feature would be to verify that the selected font
//! 	<i>has</i> a given glyph available before allowing the substitution.
//! 	The callback should return true to allow the substitution, false to
//! 	prevent it.  If no callback is provided, all substitutions are allowed.
//! @param cbdata
//!	Callback data. Will be passed to CB_AllowSub as the 2nd parameter
//! @param flags
//! \li <b>GLYPHSUB_AllowWideForms</b> Wide (or zenkaku) compatibility characters
//!	\li <b>GLYPHSUB_AllowNarrowForms</b> Narrow (or hankaku) compatibility characters
//!	\li <b>GLYPHSUB_AllowCompatForms</b> Compatibility (equivalent) forms
//!	\li <b>GLYPHSUB_AllowPositionalForms</b> Arabic/Hebrew positional forms
//!	\li <b>GLYPHSUB_AllowFractionForms</b>  Converts the three characters "1/2" 
//!		to a single fraction glyph
//! \li <b>GLYPHSUB_AllowSubscriptForms</b>
//! \li <b>GLYPHSUB_AllowSuperscriptForms</b>
//! \li <b>GLYPHSUB_AllowSmallForms</b> Small variant forms (CNS compatibility)
//! \li <b>GLYPHSUB_AllowSquareForms</b> CJK squared font variant
//! \li <b>GLYPHSUB_AllowCircleForms</b> Encircled forms
//! \li <b>GLYPHSUB_AllowFontForms</b> Font variants (e.g. a blackletter form)
//! \li <b>GLYPHSUB_AllowNoBreakForms</b> Non-breaking spaces and hyphens
//! \li <b>GLYPHSUB_AllowVerticalForms</b> Vertical presentation forms
//!
LIBEXPORT	MIUNICODE* MucDoGlyphSubstitutions (
	const MIUNICODE* instr,
	bool (*CB_AllowSub)(const UCDATA* sub, void* cbdata),
	void* cbdata,
	UINT32 flags = 0
	);
#endif	//!< c++

#define GLYPHSUB_AllowWideForms				0x00000001	
#define GLYPHSUB_AllowNarrowForms			0x00000002
#define GLYPHSUB_AllowCompatForms			0x00000004	//!< Compatibility (equivalent) forms
#define GLYPHSUB_AllowPositionalForms		0x00000008	//!< Arabic/Hebrew
#define GLYPHSUB_AllowFractionForms			0x00000010  //!< "1/2" -> single glyph
#define GLYPHSUB_AllowSubscriptForms		0x00000020
#define GLYPHSUB_AllowSuperscriptForms		0x00000040
#define GLYPHSUB_AllowSmallForms				0x00000080
#define GLYPHSUB_AllowSquareForms			0x00000100
#define GLYPHSUB_AllowCircleForms			0x00000200
#define GLYPHSUB_AllowFontForms				0x00000400
#define GLYPHSUB_AllowNoBreakForms			0x00000800
#define GLYPHSUB_AllowVerticalForms			0x00001000

//! Flip Right-To-Left text to "visual" order.
//!
//! Returns a copy of the input string which caller must free.
//! This function is symetric.  That is, flipping the same string twice
//! gives the original string.
LIBEXPORT MIUNICODE* MucFlipVisualAndLogicalOrder (
	const MIUNICODE*
	);

//! Returns the default charset for keyboard input.
//!
//! Returns 3 charset codes which can be used as hints when trying
//! to convert MIUNICODE to ISO-2022.  This is used by the input of
//! the XeText widget.
//!
//! @param charset1 Charset to use for single-byte things < 128
//! @param charset2 Charset to use for single-byte things > 128
//! @param charset3	Charset to use for multi-byte things.
//!
LIBEXPORT int MucGetDefaultCharsets (
	int* charset1, 
	int* charset2, 
	int* charset3
	);


//! Determine the direction of a glyph.
//!
//! Given a MIUNICODE string and a current direction, determine the direction
//! of the next character in the string.
//!
//! The glyphflags array is filled in by this function.
//!
//! <b>Directions</b> (both curdir and the return value)
//! \li MucGLYPHDIRECTION_LtoR		
//! \li MucGLYPHDIRECTION_RtoL	
//!
//! <b>glyphflags</b>
//! \li MucGLYPHFLAG_PartOfRtoLWord		
//! \li MucGLYPHFLAG_NonJoining		
//! \li MucGLYPHFLAG_Mark			
//!
LIBEXPORT int MucGetGlyphDirection (
	const MIUNICODE *uc, 
	int curdir, 
	UINT8* glyphflags
	);

//! Get the number of supported encodings of a given type.
//!
//! @param flags
//! \li <b>ENCODELIST_FLAG_Any</b>	Allow any encoding
//! \li	<b>ENCODELIST_FLAG_Importable</b> Allow importable encodings
//! \li	<b>ENCODELIST_FLAG_Exportable</b> Allow exportable encodings
//! \li	<b>ENCODELIST_FLAG_Both</b> Allow importanble or exportable encodings
//! \li	<b>ENCODELIST_FLAG_NoUnicode</b> Don't include MIUNICODE
LIBEXPORT int MucGetNumEncodings (
	ENCODELIST_FLAGS flags
	);

//! Given an encoding name, return the CHAR_ENCODING value.
LIBEXPORT CHAR_ENCODING MucGetEncodingFromName (
	const char* name
	);


//! Given an encoding number, return the CHAR_ENCODING value.
//!
//! This will return the Nth encoding which matches the criteria set forth
//! by the flags parameter.  Encoding numbers (as passed to this function)
//! are not fixed and can change at any time.  The only thing this function
//! should be used for is retrieving a list of encodings to present the user
//! with a choice.
//!
//! @param num The encoding number.  
//! 	
//! @param flags
//! \li <b>ENCODELIST_FLAG_Any</b>	Allow any encoding
//! \li	<b>ENCODELIST_FLAG_Importable</b> Allow importable encodings
//! \li	<b>ENCODELIST_FLAG_Exportable</b> Allow exportable encodings
//! \li	<b>ENCODELIST_FLAG_Both</b> Allow importanble or exportable encodings
//! \li	<b>ENCODELIST_FLAG_NoUnicode</b> Don't include MIUNICODE
LIBEXPORT CHAR_ENCODING MucGetEncodingFromNum (
	int num, 
	ENCODELIST_FLAGS flags
	);

//! Return the index into array of encodings for a given encoding.
//!
//! If you use MucGetNumEncodings() and MucNumToName() to iterate through
//! all the encodings of a given type for the purposes of generating a 
//! list for the user to select from, then this function is useful for
//! determining which item in that list a given encoding will be so 
//! that you can make it the default.
//!
//! @param encoding The encoding to find
//! 	
//! @param flags
//! \li <b>ENCODELIST_FLAG_Any</b>	Allow any encoding
//! \li	<b>ENCODELIST_FLAG_Importable</b> Allow importable encodings
//! \li	<b>ENCODELIST_FLAG_Exportable</b> Allow exportable encodings
//! \li	<b>ENCODELIST_FLAG_Both</b> Allow importanble or exportable encodings
//! \li	<b>ENCODELIST_FLAG_NoUnicode</b> Don't include MIUNICODE
//!
//! @return Index into array or -1 if encoding not found.
LIBEXPORT int MucGetEncodingNum (
	CHAR_ENCODING encoding,
	ENCODELIST_FLAGS flags
	);

//! Return an encoding name. 
//!
//! Once an encoding is named, the name is set in stone and can be written
//! to files without fear of the name changing.  The actual name presented
//! to the user is looked up in messages.txt by calling MucNameToDescUC()
//!
//! Do NOT free the return value from this function
LIBEXPORT const char* MucGetName (
	CHAR_ENCODING encoding
	);		

//! Given a string which may be in either JIS, Shift-JIS, or EUC encoding,
//! try to determine which it is.  The "encoding" string passed in must
//! be big enough to hold the resulting encoding name "Shift-JIS" is the
//! longest string it can return.
LIBEXPORT int MucGuessJapaneseEncoding (
	const UINT8 *in, 
	char *encoding
	); 

//! Determine if a MIUNICODE string has right-to-left text in it.
LIBEXPORT int MucHasRightToLeftTextUC (
	const MIUNICODE *in
	);

//! Determine if an ISO-2022 string has right-to-left text in it.
LIBEXPORT int MucHasRightToLeftTextISO2022 (
	const char *in
	);

//! Return the description of an encoding given the name (char*).
//!
//! This looks up the incoding description from messages.txt.
//! You should free the return value when you're done with it.
//! Don't call this.  Calle MucNameToDescUC() instead
//! @see MucNameToDescUC()
LIBEXPORT char *MucNameToDesc (
	const char* name
	);	

//! Return the description of an encoding given the name (MIUNICODE*).
//!
//! This looks up the incoding description from messages.txt.
//! You should free the return value when you're done with it.
LIBEXPORT MIUNICODE *MucNameToDescUC (
	const char* name
	);	

//! The opposite of MucNumToName().
LIBEXPORT int MucNameToNum (
	const char* name, 
	ENCODELIST_FLAGS flags
	);

//! Determine the primary character set for a given encoding.
LIBEXPORT int MucNameToCharset (
	const char* name , 
	ENCODELIST_FLAGS flags
	);

//! Return the description of an encoding given the number (char*).
//!
//! This looks up the incoding description from messages.txt.
//! You should free the return value when you're done with it.
//! Don't call this.  Calle MucNumToDescUC() instead
//! @see MucNumToDescUC()
LIBEXPORT char *MucNumToDesc (
	int num, 
	ENCODELIST_FLAGS flags
	);

//! Return the description of an encoding given the number (MIUNICODE*).
//!
//! This looks up the incoding description from messages.txt.
//! You should free the return value when you're done with it.
LIBEXPORT MIUNICODE *MucNumToDescUC (
	int num, 
	ENCODELIST_FLAGS flags
	);

//! Convert an encoding number to its name.
//!
//! Combines the following two functions
//! @see MucGetEncodingFromNum()
//! @see MucGetName()
//!
//! Do not free the output from this function
LIBEXPORT const char *MucNumToName (
	int num, 
	ENCODELIST_FLAGS flags
	);	

//! Don't use this function.
LIBEXPORT int MucQuickConvToUnicode (
	int encodetype, 
	const void *in, 
	MIUNICODE **ucp
	);

//! Don't use this function.
LIBEXPORT int MucQuickConvToISO2022 (
	int encodetype, 
	const void *in, 
	char **outp
	);

//! Reset an Muc handle.
LIBEXPORT void MucReset (
	void* handle
	);

//! Determine the number of characters in an ISO-2022 string.
//!
//! Since ISO-2022 can consist of single-byte and two-byte characters
//! and escape codes, a simple strlen() won't give you an accurate
//! length.  This function gives you the same result you'd get if you
//! converted the string to MIUNICODE and called ucstrlen() on the result,
//! but without going to the actual work.
LIBEXPORT int MucStrLenISO2022 (
	const void *vinbuf
	);

#define MUC_THAI_OF	1
#define MUC_THAI_TTF	2
LIBEXPORT MIUNICODE *MucThaiShiftTonesAndVowels (
	const MIUNICODE* str, 
	UINT32 fontencoding		//!< MUC_THAI_OF or MUC_THAI_TTF
	);

#ifdef __cplusplus
//! Does Indic glyph substitution based on various rules.
//!
//! script should be one of the following:
//!	 SCRIPTTAG_Default	(See below)
//!	 SCRIPTTAG_beng 	(Bengali)
//!	 SCRIPTTAG_deva 	(Devanagari)
//!	 SCRIPTTAG_gujr 	(Gujarati)
//!	 SCRIPTTAG_knda 	(Kannada)
//!	 SCRIPTTAG_mlym 	(Malayalam)
//!	 SCRIPTTAG_orya 	(Oriya)
//!	 SCRIPTTAG_punj 	(punjabi)
//!	 SCRIPTTAG_taml 	(Tamil)
//!	 SCRIPTTAG_telu 	(Telugu)
//!
//! If script is SCRIPTTAG_Default, currently it will fall back on Devanagari,
//! but eventually it will try to find the current system script and use that.
//!
//! The len parameter is used to avoid an extra ucstrlen() call.  If the
//! caller already knows the length of instr, pass it. If not, just omit the
//! parameter.
//!
//! The caller is responsible for freeing outstr
//!
//! The flag bApplyRephHack is just that, a hack.  In order to correctly process
//! the GSUB tables, we need to only apply the 'rphf' table if we have reordered
//! a reph+halant+consonant combination to consonant+reph+halant.  In order to 
//! do that, we need to somehow distinguish between text where we moved the reph
//! and text where it was already there.
//! To do this, if this flag is set, we change the reph to a fake Unicode 
//! character, one that we chose by subtracting 0x30 from the reph glyph.  In 
//! all the Indic encodings, this falls on an unused code point (for example, 
//! 0x0930 becomes 0x0900).  The caller knows to look for this and converts it 
//! back to the real reph glyph, but sets a flag telling us to process the 
//! 'rphf' table on it.
//! 
LIBEXPORT ERRVALUE MucIndicGlyphSubstitutions (
	const MIUNICODE* instr,
	MIUNICODE** outstr,
	SCRIPTTAG script = SCRIPTTAG_Default,
	INT32 len = -1,		//!< Length of instr if known (just to avoid extra ucstrlen())
	bool bApplyRephHack = false
	);
#endif

//!	Returns true if ch is a Thai tone character
LIBEXPORT bool MucIsThaiTone (
	MIUNICODE ch
	);

//!	Returns true if ch is a Thai vowel character
LIBEXPORT bool MucIsThaiVowel (
	MIUNICODE ch
	);

//!	Returns true if ch is a Thai vowel character that sits above the consonant
LIBEXPORT bool MucIsThaiUpperVowel (
	MIUNICODE ch
	);

//!	Returns true if ch is a Thai vowel character that sits below the consonant
LIBEXPORT bool MucIsThaiLowerVowel (
	MIUNICODE ch
	);

//! Convert a single MIUNICODE character to a given character set.
//!
//! Returns 0 if the character is not available in the given charset
LIBEXPORT int MucUnicodeToCharset (
	int val, 
	int charset
	);

#ifdef WIN32
//! Set function override to get current language ID (WIN32 only).
LIBEXPORT void MucWinSetFuncGetLanguageID (
	UINT16 (*pfGetLanguageID)(void)
	);

//! Set function override to get current Charset ID (WIN32 only).
LIBEXPORT void MucWinSetFuncGetCharsetID (
	UINT16 (*pfGetLanguageID)(void)
	);
#endif


#if defined(__cplusplus)
}
#endif

#define MUCEVENT_ComposeOn    	0x01
#define MUCEVENT_ComposeOff      0x02
#define MUCEVENT_ComposeToggle   (MUCEVENT_ComposeOn|MUCEVENT_ComposeOff)

#undef LIBEXPORT 

#endif	// INC_MI32_MUCDEFNS_H
