/*************************************************************************
 *
 *  mi32/mxgrlist.h - Graphical List Widget
 *
 * \if NODOC
 * $Id: mxgrlist.h_v 1.17 2011/11/03 14:33:37 mju Exp $
 *
 * $Log: mxgrlist.h_v $
 * Revision 1.17  2011/11/03 14:33:37  mju
 * Use mcb hdr.
 *
 * Revision 1.16  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.15  2002/09/18 20:11:11  dwilliss
 * Added members to motion notify structure to pass root window-relative
 * coordinates of motion notify event
 *
 * Revision 1.14  2000/07/05 18:08:07  sparsons
 * Genitor documentation.
 *
 * Revision 1.13  1999/05/07 21:20:17  mju
 * Header restruct.
 *
 * Revision 1.12  1999/04/08  15:53:58  mju
 * Fix inclusion guard name.
 *
 * Revision 1.11  1999/01/05  19:02:06  scowan
 * Swapped multiple keywords
 *
 * Revision 1.10  1998/09/24 17:18:04  scowan
 * C++ linkage fix.
 *
 * Revision 1.9  1998/09/04  14:29:24  vdronov
 * *** empty log message ***
 *
 * Revision 1.8  1998/09/03  14:54:07  vdronov
 * *** empty log message ***
 *
 * Revision 1.7  1998/07/16  22:28:23  VDRONOV
 * added new function
 *
 * Revision 1.6  1998/03/10  16:21:43  VDRONOV
 * add single\multiple mode.
 *
 * Revision 1.5  1998/02/07  00:21:58  VDRONOV
 * add few functions.
 *
 * Revision 1.4  1997/10/22  14:26:59  VDRONOV
 * clean some control characters like \015.
 *
 * Revision 1.3  1997/10/21  23:05:41  VDRONOV
 * *** empty log message ***
 *
 * Revision 1.1  1997/07/28  17:59:40  VDRONOV
 * Initial revision
 *

 *
 * \endif
*************************************************************************/

#ifndef INC_MI32_MXGRLIST_H
#define INC_MI32_MXGRLIST_H

#ifndef	INC_MI32_XDEFNS_H
	#include <mi32/xdefns.h>
#endif

#ifndef	INC_MI32_MCB_H
	#include <mi32/mcb.h>
#endif

/* Callback reasons for MxGraphList widget */

#define MxGraphList_OwnDrawReason			0x0001
#define MxGraphList_ButtonPressReason		0x0002
#define MxGraphList_ButtonReleaseReason	0x0004
#define MxGraphList_ButtonReason				0x0006
#define MxGraphList_MotionNotifyReason		0x0008
#define MxGraphList_KeyPressReason			0x0010
#define MxGraphList_KeyReleaseReason		0x0020
#define MxGraphList_KeyReason					0x0030
#define MxGraphList_HeaderDrawReason		0x0040

#define MxGraphList_SingleSelection			0
#define MxGraphList_MultipleSelection		1

struct MxGraphListDrawStruct {
	Drawable window;
	Display *display;
	GC gc;
	int x;
	int y;
	int width;
	int height;
	};

struct MxGraphListButtonStruct {
	int px;
	int py;
	unsigned int state;
	unsigned int button;
	int x;
	int y;
	int width;
	int height;
	};

struct MxGraphListMotionStruct {
	int px;
	int py;
	unsigned int state;
	unsigned int is_hint;
	int x;
	int y;
	int width;
	int height;
	int x_root;
	int y_root;
	};

struct MxGraphListKeyStruct {
	unsigned int state;
	KeySym keysym;
	};

struct MxGraphListCallbackStruct {
	UINT32 reason;
	int item;
	int item_position;
	int item_state;
	MxGraphListDrawStruct	draw;
	MxGraphListButtonStruct button;
	MxGraphListMotionStruct motion;
	MxGraphListKeyStruct		key;
	};

#if defined(__cplusplus)
extern "C" {
#endif

//! Add callback to the GraphList widget.
//!
//! Reasons:
//! 	MxGraphList_OwnDrawReason		 
//!		MxGraphList_ButtonPressReason	  
//!		MxGraphList_ButtonReleaseReason  			
//!		MxGraphList_ButtonReason		  
//!		MxGraphList_MotionNotifyReason	  			
//!		MxGraphList_KeyPressReason		  
//!		MxGraphList_KeyReleaseReason	  			
//!		MxGraphList_KeyReason			  
//!		MxGraphList_HeaderDrawReason
int MxGraphListAddCallback (
	Widget w,												//!< "MxGraphList" widget returned by MxGraphListCreate function	
	void (*func)(McbAnyCallbackStruct *,void *),	//! Callback function
	void *data,												//!< Data passed to the callback function
	UINT32 reason											//!< Callback reason 
	);

//! Get drawing area widget.
//!
//! @return DrawingArea widget or NULL if error.
Widget MxGraphListGetDrawingAreaWidget (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Create the GraphList widget.
//!
//! @return MxGraphList" widget returned or NULL if error.
Widget MxGraphListCreate (
	Widget parent,								//!< Parent widget
	int Width,									//!< Initial width of the drawing area 
	int Height,									//!< Initial height of the drawing area
	void *userdata,							//!< Userdata (not used)
	UINT32 flags,								//!< Flags
	Arg *arg,									//!< Attachments for the "MxGraphList" widget
	int n											//!< Number of elements in arg array
	);

//! Get an item geometrical size.
void MxGraphListGetItemGeometry (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int *Width, 								//!< Item's width in pixels returned
	int *Height									//!< Item's height in pixels returned
	);

//! Set list an item geometrical size.
void MxGraphListSetItemGeometry (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int Width, 									//!< Item's width in pixels
	int Height									//!< Item's height in pixels
	);

//! Get header's height.
void MxGraphListGetHeaderHeight (
	Widget w,									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int *height									//!< Header's height in pixels returned 
	);

//! Set header's height.
void MxGraphListSetHeaderHeight (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int height									//!< Header's height in pixels 
	);

//! Get visible items count.
void MxGraphListGetVisibleItemsCount (
	Widget w,									//!< "MxGraphList" widget returned by MxGraphListCreate function 
	int *count									//!< Visible items count returned 
	);

//! Set visible items count.
void MxGraphListSetVisibleItemsCount (
	Widget w,									//!< "MxGraphList" widget returned by MxGraphListCreate function 
	int count									//!< Visible items count
	);

//! Draw specified item.
void MxGraphListDrawPos (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int position								//!< Position in list	
	);

//! Redraw the GraphList widget.
void MxGraphListRedraw (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Enable/disable the GraphList redrawing.
void MxGraphListEnableRedraw (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int EnableRedraw							//!< Enable redraw if not 0,	disable redraw if 0	
	);

//! Get items count.
//!
//! @return Item count or error code if < 0.
int MxGraphListGetItemCount (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Add an item to the GraphList.
void MxGraphListAddItem (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item, 									//!< Item to be added to list
	int position								//!< Position of the new item in the list (started from 1, 0 means to end)
	);

//! Add items to the GraphList.
void MxGraphListAddItems (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int *items, 								//!< Pointer to the items to be added to the list
	int item_count, 							//!< Number of items in items
	int position								//!< Position of the first new item in the list (started from 1, 0 means to end)
	);

//! Delete all items from GraphList.
void MxGraphListDeleteAllItems (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Delete an item from the GraphList.
void MxGraphListDeleteItem (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item										//!< Item to be deleted from list (all items match specified item)
	);

//! Delete items from the GraphList.
void MxGraphListDeleteItems (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int *items, 								//!< Pointer to the items to be deleted from list (all items match specified items)
	int item_count								//!< Number of elements in items
	);

//! Delete items from the GraphList starting at the given position.
void MxGraphListDeleteItemsPos (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item_count, 							//!< Number of items to be deleted
	int position								//!< Position in the list of the first item to be deleted
	);

//! Delete item from the GraphList at a specified position.
void MxGraphListDeletePos (
	Widget w,									//!< "MxGraphList" widget returned by MxGraphListCreate function 
	int position								//!< Position of the item to be deleted
	);

//! Deselect all items.
void MxGraphListDeselectAllItems (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Deselect the specified item.
void MxGraphListDeselectItem (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item										//!< Item to be deselected from list (all items match specified item)
	);

//! Deselect an item at a specified position.
void MxGraphListDeselectPos (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int position								//!< Position of the item to be deselected
	);

//! Select all items. 
void MxGraphListSelectAllItems (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Invert selection status for every item.
void MxGraphListInvertSelect (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Select an item.
void MxGraphListSelectItem (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item										//!< Item to be selected in list (all items match specified item)
	);

//! Select an item at a specified position.
void MxGraphListSelectPos (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int position								//!< Position of the item to be selected
	);

//! Make an item the last visible item.
void MxGraphListSetBottomItem (
	Widget w,									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item										//!< Item to be last visible in list	(first item matches specified item)
	);

//! Make an item at the specified position the last visible item.
void MxGraphListSetBottomPos (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int position								//!< Position of the item to be selected
	);

//! Make an item the first visible item.
void MxGraphListSetItem (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item										//!< Item to be first visible in list (first item matches specified item)
	);

//! Make an item at the specified position the first visible item.
void MxGraphListSetPos (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int position								//!< Position of the item to be selected
	);

//! Get an array of positions where a specified item is found.
void MxGraphListGetMatchPos (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item, 									//!< Item to be matched
	int **position_list, 					//!< Array of positions (allocated and returned)
	int *position_count						//!< Number of elements in the position_list
	);

//! Get an array of the positions of the selected items.
//!
//! @return 1 if the GraphList has any selected elements, 0 if the GraphList doesn't have any selected elements, <0 if error.
int MxGraphListGetSelectedPos (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int **position_list, 					//!< Array of positions of the selected items (allocated and returned)
	int *postion_count						//!< Number of elements in the position_list
	);

//! Get selected items count.
//!
//! @return Number of selected elements or error code if < 0.
int MxGraphListGetSelectedItemCount (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Check if a specified item is in the GraphList.
//!
//! @return 1 if the specified item is present in the list.
int MxGraphListItemExists (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item										//!< Item whose presence is checked
	);

//! Get the position of an item in the GraphList.
//!
//! @return Position in the list of the first instance of the specified item or error code if < 0.
int MxGraphListItemPos (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item										//!< Item whose position is returned
	);

//! Get an item at a specified position.
//!
//! @return Item or error code if < 0.
int MxGraphListGetItem (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int item										//!< Position in the list
	);

//! Check if the item at a specified position is selected.
//!
//! @return 1 if item is selected.
int MxGraphListPosSelected (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int position								//!< Position in the list
	);

//! Get the position of the item at the location cursor.
//!
//! @return Position in the list.
int MxGraphListGetKbdItemPos (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Set the location cursor at a specified position.
//!
//! @return 1 if operation is succesful.
int MxGraphListSetKbdItemPos (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int position								//!< Position in the list
	);

//! Grab control of the pointer.
int MxGraphListGrabPointer (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Release the pointer.
int MxGraphListUngrabPointer (
	Widget w										//!< "MxGraphList" widget returned by MxGraphListCreate function
	);

//! Set selection mode.
//!
//! @return 1 if operation is succesful.
//!
//! Selection modes:
//! 	MxGraphList_SingleSelection					
//!		MxGraphList_MultipleSelection
int MxGraphListSetSelectionMode (
	Widget w, 									//!< "MxGraphList" widget returned by MxGraphListCreate function
	int mode										//!< Selection mode
	);

#if defined(__cplusplus)
}
#endif

#endif		//!<  #ifndef	INC_MI32_MXGRLIST_H 

