/**
 * \file mi32/ucchar.h
 * \brief Unicode "character" functions
 *
 * \if NODOC
 * $Id: ucchar.h_v 1.3 2011/09/06 21:34:19 mju Exp $
 *
 * $Log: ucchar.h_v $
 * Revision 1.3  2011/09/06 21:34:19  mju
 * Rename macro.
 *
 * Revision 1.2  2008/11/17 14:00:48  mju
 * Add isdigit0to9.
 *
 * Revision 1.1  2008/11/14 15:29:58  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_UCCHAR_H
#define	INC_MI32_UCCHAR_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif

//!\addtogroup UCCHAR Unicode Character Functions
//!@{

//! These are the general category codes from column 2 of the
//! UnicodeData.txt.  The numbers are arbitrarily assigned by
//! me.  UnicodeData.txt has the 2-letter codes.
//! You may be asking why I defined the values for all the
//! enums when the compiler would do that for me...See, this
//! way it would be a pain to insert something into the middle
//! and renumber them, wouldn't it?  Good!  don't do that.
//! ucdata.ref has the numbers and if you renumber them, you'll
//! break things.
enum UCCAT {
	UCCAT_Cn = 0,			//!< Other , Not assigned (norm)
	UCCAT_Lu = 1,			//!< Letter, Uppercase (norm)
	UCCAT_Ll = 2,			//!< Letter, Lowercase (norm)
	UCCAT_Lt = 3,			//!< Letter, Titlecase (norm)
	UCCAT_Lm = 4,			//!< Letter, Modifier
	UCCAT_Lo = 5,			//!< Letter, Other
	UCCAT_Mn = 6,			//!< Mark, non-spacing (norm)
	UCCAT_Mc = 7,			//!< Mark, spacing combining (norm)
	UCCAT_Me = 8,			//!< Mark, encolsing (norm)
	UCCAT_Nd = 9,			//!< Number Decimal digit (norm)
	UCCAT_Nl = 10,			//!< Number Letter (norm)
	UCCAT_No = 11,			//!< Number Other (norm)
	UCCAT_Zs = 12,			//!< Separator, space (norm)
	UCCAT_Zl = 13,			//!< Separator, line (norm)
	UCCAT_Zp = 14,			//!< Separator, paragraph (norm)
	UCCAT_Cc = 15,			//!< Other , control (norm)
	UCCAT_Cf = 16,			//!< Other , format (norm)
	UCCAT_Cs = 17,			//!< Other , surrogate (norm)
	UCCAT_Co = 18,			//!< Other , Private use (norm)
	UCCAT_Pc = 20,			//!< Punctuation, connector
	UCCAT_Pd = 21,			//!< Punctuation, Dash
	UCCAT_Ps = 22,			//!< Punctuation, Open
	UCCAT_Pe = 23,			//!< Punctuation, Close
	UCCAT_Pi = 24,			//!< Punctuation, Initial quote
	UCCAT_Pf = 25,			//!< Punctuation, final quote
	UCCAT_Po = 26,			//!< Punctuation, other
	UCCAT_Sm = 27,			//!< Symbol, math
	UCCAT_Sc = 28,			//!< Symbol, currency
	UCCAT_Sk = 29,			//!< Symbol, modifier
	UCCAT_So = 30			//!< Symbol, other
	};

//! Flags for the above which can be ORd together
enum UCCATFLAGS {
	UCCATFLAG_Cn = (1 << UCCAT_Cn),			//! Other , Not assigned (norm)
	UCCATFLAG_Lu = (1 << UCCAT_Lu),			//! Letter, Uppercase (norm)
	UCCATFLAG_Ll = (1 << UCCAT_Ll),			//! Letter, Lowercase (norm)
	UCCATFLAG_Lt = (1 << UCCAT_Lt),			//! Letter, Titlecase (norm)
	UCCATFLAG_Lm = (1 << UCCAT_Lm),			//! Letter, Modifier
	UCCATFLAG_Lo = (1 << UCCAT_Lo),			//! Letter, Other
	UCCATFLAG_Mn = (1 << UCCAT_Mn),			//! Mark, non-spacing (norm)
	UCCATFLAG_Mc = (1 << UCCAT_Mc),			//! Mark, spacing combining (norm)
	UCCATFLAG_Me = (1 << UCCAT_Me),			//! Mark, encolsing (norm)
	UCCATFLAG_Nd = (1 << UCCAT_Nd),			//! Number Decimal digit (norm)
	UCCATFLAG_Nl = (1 << UCCAT_Nl),			//! Number Letter (norm)
	UCCATFLAG_No = (1 << UCCAT_No),			//! Number Other (norm)
	UCCATFLAG_Zs = (1 << UCCAT_Zs),			//! Separator, space (norm)
	UCCATFLAG_Zl = (1 << UCCAT_Zl),			//! Separator, line (norm)
	UCCATFLAG_Zp = (1 << UCCAT_Zp),			//! Separator, paragraph (norm)
	UCCATFLAG_Cc = (1 << UCCAT_Cc),			//! Other , control (norm)
	UCCATFLAG_Cf = (1 << UCCAT_Cf),			//! Other , format (norm)
	UCCATFLAG_Cs = (1 << UCCAT_Cs),			//! Other , surrogate (norm)
	UCCATFLAG_Co = (1 << UCCAT_Co),			//! Other , Private use (norm)
	UCCATFLAG_Pc = (1 << UCCAT_Pc),			//! Punctuation, connector
	UCCATFLAG_Pd = (1 << UCCAT_Pd),			//! Punctuation, Dash
	UCCATFLAG_Ps = (1 << UCCAT_Ps),			//! Punctuation, Open
	UCCATFLAG_Pe = (1 << UCCAT_Pe),			//! Punctuation, Close
	UCCATFLAG_Pi = (1 << UCCAT_Pi),			//! Punctuation, Initial quote
	UCCATFLAG_Pf = (1 << UCCAT_Pf),			//! Punctuation, final quote
	UCCATFLAG_Po = (1 << UCCAT_Po),			//! Punctuation, other
	UCCATFLAG_Sm = (1 << UCCAT_Sm),			//! Symbol, math
	UCCATFLAG_Sc = (1 << UCCAT_Sc),			//! Symbol, currency
	UCCATFLAG_Sk = (1 << UCCAT_Sk),			//! Symbol, modifier
	UCCATFLAG_So = (1 << UCCAT_So), 			//! other
	UCCATFLAG_Mi = (1 << 31)					//! Mirroring
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(UCCATFLAGS)
#endif
		
//! Unicode Bidirectional category from column 4 of UnicodeData.txt
//! See details on UCCATEGORY enum above.  These are summarized in
//! chapter 3 of the Unicode Standard
enum BIDITAG {
	BIDITAG_L	= 0,			//!< Left-to-right
	BIDITAG_LRE	= 1,			//!< Left-to-right Embedding
	BIDITAG_LRO	= 2,			//!< Left-to-right Override
	BIDITAG_R	= 3,			//!< Right-to-left
	BIDITAG_AL	= 4,			//!< Right-to-left arabic
	BIDITAG_RLE	= 5,			//!< Right-to-left embedding
	BIDITAG_RLO	= 6,			//!< Right-to-left override
	BIDITAG_PDF	= 7,			//!< Pop Directional Format
	BIDITAG_EN	= 8,			//!< European Number
	BIDITAG_ES	= 9,			//!< European Number separator
	BIDITAG_ET	= 10,			//!< European Number Terminator
	BIDITAG_AN	= 11,			//!< Arabic Number 
	BIDITAG_CS	= 12,			//!< Common Number Separator
	BIDITAG_NSM	= 13,			//!< Non Spacing Mark
	BIDITAG_BN	= 14,			//!< Boundary Neutral
	BIDITAG_B	= 15,			//!< Paragraph Separator
	BIDITAG_WS	= 16,			//!< White Space
	BIDITAG_ON	= 17			//!< Other Neutrals
	};

//! Flags for the above which can be ORd together
enum BIDIFLAGS {
	BIDIFLAG_L 		= (1 << BIDITAG_L),			//! Left-to-right
	BIDIFLAG_LRE 	= (1 << BIDITAG_LRE),		//! Left-to-right Embedding
	BIDIFLAG_LRO 	= (1 << BIDITAG_LRO),		//! Left-to-right Override
	BIDIFLAG_R 		= (1 << BIDITAG_R),			//! Right-to-left
	BIDIFLAG_AL 	= (1 << BIDITAG_AL),			//! Right-to-left arabic
	BIDIFLAG_RLE 	= (1 << BIDITAG_RLE),		//! Right-to-left embedding
	BIDIFLAG_RLO 	= (1 << BIDITAG_RLO),		//! Right-to-left override
	BIDIFLAG_PDF 	= (1 << BIDITAG_PDF),		//! Pop Directional Format
	BIDIFLAG_EN 	= (1 << BIDITAG_EN),			//! European Number
	BIDIFLAG_ES 	= (1 << BIDITAG_ES),			//! European Number separator
	BIDIFLAG_ET 	= (1 << BIDITAG_ET),			//! European Number Terminator
	BIDIFLAG_AN 	= (1 << BIDITAG_AN),			//! Arabic Number 
	BIDIFLAG_CS 	= (1 << BIDITAG_CS),			//! Common Number Separator
	BIDIFLAG_NSM 	= (1 << BIDITAG_NSM),		//! Non Spacing Mark
	BIDIFLAG_BN 	= (1 << BIDITAG_BN),			//! Boundary Neutral
	BIDIFLAG_B 		= (1 << BIDITAG_B),			//! Paragraph Separator
	BIDIFLAG_WS		= (1 << BIDITAG_WS),			//! White Space
	BIDIFLAG_ON		= (1 << BIDITAG_ON)			//! Other Neutrals
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(BIDIFLAGS)
#endif


//! Determine of a Unicode character has a given property.
//! The property flags are listed in ucstring.h.
//! (the name is derived from isprop() from ctype.h)
//! There are two sets of flags because there are so many
//! possibilites with Unicode.  You won't usually use this
//! function directly.  
LIBEXPORT bool ucisprop (
	MIUNICODE c,
	UINT32 uccatflags
	);
 
LIBEXPORT bool ucisdecompbidiprop(
	MIUNICODE c,
	UINT32 decompflags
	);
 
LIBEXPORT bool ucisbidiprop (
	MIUNICODE c,
	UINT32 bidiflags
	);

LIBEXPORT MIUNICODE uctolower (
	MIUNICODE c
	);

LIBEXPORT MIUNICODE uctoupper (
	MIUNICODE c
	);

//! Alpha or Digit
inline bool ucisalnum (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Lu | UCCATFLAG_Ll | UCCATFLAG_Lm | UCCATFLAG_Lo | UCCATFLAG_Lt | UCCATFLAG_Nd);
	}

//! Alpha
inline bool ucisalpha (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Lu | UCCATFLAG_Ll | UCCATFLAG_Lm | UCCATFLAG_Lo | UCCATFLAG_Lt);
	}

//! Control
inline bool uciscntrl (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Cc | UCCATFLAG_Cf);
	}

//! Determine if digit 0 to 9.
inline bool ucisdigit0to9 (MIUNICODE c) {
	return (c >= (MIUNICODE)'0' && c <= (MIUNICODE)'9');
	}

//! Digit (any digit, not just 0-9)
inline bool ucisdigit2 (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Nd);
	}

//! Space
inline bool ucisspace (MIUNICODE c) {
	// Following line is to conform to the ANSI version of isspace()
	if ((c >= 0x09 && c <= 0x0D) || c == 0x20) return (true); 
	return ucisprop(c, UCCATFLAG_Zs);
	}

//! Space 
inline bool ucisblank (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Zs);
	}

inline bool ucispunct (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Pc | UCCATFLAG_Pd | UCCATFLAG_Ps | UCCATFLAG_Pe | UCCATFLAG_Po | UCCATFLAG_Pi | UCCATFLAG_Pf | UCCATFLAG_Po);
	}

inline bool ucisupper (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Lu);
	}

inline bool ucislower (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Ll);
	}

inline bool ucistitle (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Lt);
	}

inline bool ucisisocntrl (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Cc);
	}

inline bool ucisfmtcntrl (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Cf);
	}

inline bool ucissymbol (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Sm | UCCATFLAG_Sc | UCCATFLAG_So | UCCATFLAG_Sk);
	}

//! Number (digit, letter form, other)
inline bool ucisnumber (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Nd | UCCATFLAG_No | UCCATFLAG_Nl);
	}

//! Mark, Non spacing
inline bool ucisnonspacing (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Mn);
	}

//! Punctuation, Open
inline bool ucisopenpunct (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Ps);
	}

//! Punctuation, Close
inline bool ucisclosepunct (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Pe);
	}

//! Punctuation, Initial
inline bool ucisinitialpunct (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Pi);
	}

//! Punctuation, Final
inline bool ucisfinalpunct (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Pf);
	}

//! Strong Right-to-Left Directionality
inline bool ucisrtl (MIUNICODE c) {
	return ucisbidiprop(c, BIDIFLAG_R | BIDIFLAG_AL);
	}

//! Strong Left-to-Right Directionality
inline bool ucisltr (MIUNICODE c) {
	return ucisbidiprop(c, BIDIFLAG_L);
	}

//! Strong R-to-L or L-to-R Directionality
inline bool ucisstrong (MIUNICODE c) {
	return ucisbidiprop(c, BIDIFLAG_L | BIDIFLAG_R | BIDIFLAG_AL);
	}

//! Weak Directionality
inline bool ucisweak (MIUNICODE c) {
	return ucisbidiprop(c, BIDIFLAG_EN | BIDIFLAG_ES | BIDIFLAG_ET | BIDIFLAG_AN | BIDIFLAG_CS);
	}

//! Netutral Directionality
inline bool ucisneutral (MIUNICODE c) {
	return ucisbidiprop(c, BIDIFLAG_WS | BIDIFLAG_ON);
	}


//! Mark (any)
inline bool ucismark (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Mn | UCCATFLAG_Mc | UCCATFLAG_Me);
	}

//! Letter, modifier
inline bool ucismodif (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Lm);
	}

//! Punctuation, Connecting
inline bool ucisconnect (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Pc);
	}

//! Punctuation, Dash
inline bool ucisdash (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Pd);
	}

//! Symbol, math
inline bool ucismath (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Sm);
	}

//! Symbol, Currency
inline bool uciscurrency (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Sc);
	}

//! Symbol, modifier
inline bool ucismodifsymbol (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Sk);
	}

//! Mark, non-spacing
inline bool ucisnsmark (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Mn);
	}

//! Mark, spacing
inline bool ucisspmark (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Mc);
	}

//! Mark, enclosing
inline bool ucisenclosing (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Me);
	}

//! Seperator, Line
inline bool ucislsep (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Zl);
	}

//! Seperator, Paragraph
inline bool ucispsep (MIUNICODE c) {
	return ucisprop(c, UCCATFLAG_Zp);
	}

//! Han glyph
inline bool ucishan (MIUNICODE c) {
	return ((c >= 0x4e00 && c <= 0x9fff) || (c >= 0xf900 && c <= 0xfaff));
	}

//! Hangul glyph
inline bool ucishangul (MIUNICODE c) {
	return (c >= 0xac00 && c <= 0xd7ff);
	}

//!@}

#undef LIBEXPORT 

#endif	// INC_MI32_UCCHAR_H
