/******************************************************************************
 * \file sml/restriction.h
 * \brief SML Class usage restrictions
 *
 * \if NODOC
 * $Log: restriction.h_v $
 * Revision 1.3  2011/10/11 21:08:46  mju
 * Remove redundant hdrs.
 *
 * Revision 1.2  2007/05/17 19:42:47  dwilliss
 * Changed to allow multiple products
 *
 * Revision 1.1  2007/05/04 22:46:55  dwilliss
 * Initial revision
 * \endif
*****************************************************************************/

#ifndef INC_SML_RESTRICTION_H
#define INC_SML_RESTRICTION_H

#ifndef	INC_SML_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

#ifndef	INC_SML_ENUMSET_H
	#include <mi32/enumset.h>
#endif

#ifndef	INC_SML_USAGE_H
	#include <sml/usage.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

namespace SML {


/*****************************************************************************/
class RVCSYSLIBCLASSEXPORT RESTRICTION {
	public:
		//! Construct an SML::RESTRICTION
		//! The default is to completely unrestricted
		RESTRICTION (
			) { m_Usage.SetAll(); }

		//! Add a particular SML::USAGE to the list of allowed usags.
		//! Note, since the RESTRICTION is set to allow all on initialization,
		//! you need to call ClearUsage before AddUsage. 
		void AllowUsage (
			SML::USAGE usage
			) { m_Usage += usage; }

		//! Disallow all SML::USAGEs.
		//! Call this before calling AddUsage
		void ClearUsage (
			) { m_Usage.Clear(); }

		//! Remove a particular SML::USAGE from the list of allowed usags.
		void DisallowUsage (
			SML::USAGE usage
			) { m_Usage -= usage; }

		//! Determine if the restriction is allowed under the current
		//! product
		bool IsAllowed (
			MIPRODUCTCODE CurrentProduct
			) const;

		//! Determine if the restriction is allowed under the current
		//! script type
		bool IsAllowed (
			SML::USAGE ScriptUsage
			) const;

		//! Determine if the restriction is allowed under the current
		//! context and script type
		bool IsAllowed (
			MIPRODUCTCODE CurrentProduct,
			SML::USAGE ScriptUsage
			) const;

		//! Set the minimum product under which this restriction is allowed
		//! By default, all products are allowed.  If you add any, then only
		//! products added (and those implied by added products) will be allowed.
		//! For example, if you add TNTview, then TNTmips is impled.  If you
		//! Add TNTmips but not TNTview, then it will not be allowed in TNTview.
		void AddProduct (
			MIPRODUCTCODE MinimumProduct
			);

	private:
		SIMPLE_ARRAY<MIPRODUCTCODE> m_AllowedProducts;
		ENUMSET<SML::USAGE, SML::USAGE_LAST> m_Usage;

	};

} // end of SML namespace


#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPOR

#endif		//  INC_SML_RESTRICTION_H
