/*****************************************************************************
*	\file <soa/geonames.h>
*	\brief GEONAMES definitions
*
*	\if NODOC
*	$Id: geonames.h_v 1.3 2009/05/18 16:04:58 bcolombe Exp $
*
*	$Log: geonames.h_v $
*	Revision 1.3  2009/05/18 16:04:58  bcolombe
*	Added extents to record
*
*	Revision 1.2  2008/03/25 14:01:32  bcolombe
*	Added accruacy to record
*
*	Revision 1.1  2008/03/18 18:01:41  bcolombe
*	Initial revision
*
*	\endif
*/

#ifndef INC_SOA_GEONAMES_H
#define INC_SOA_GEONAMES_H

#ifndef INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef INC_MI32_POLYLINE_H
#include <mi32/polyline.h>
#endif

#ifndef INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

namespace SOA {

class GEONAMES {
	public:
	
		//! Default constructor
		GEONAMES ();

		//! Destructor
		~GEONAMES ();

		//! Result
		class RECORD {
			public:
				//! Default constructor
				RECORD();

				//! Copy constructor
				RECORD(const RECORD& rhs);

				//! Destructor
				~RECORD();

				INT32 GetAccuracy() {return m_accuracy;}

				void GetCountryName(MISTRING& countryname) {countryname = m_countryname;}

				void GetExtents(DRECT2D& extents) {extents = m_extents;}
				
				void GetLocation(DPOINT2D& location) {location = m_location;}

				void GetName(MISTRING& name) {name = m_name;}

				void SetAccuracy(INT32 accuracy) {m_accuracy = accuracy;}

				void SetCountryName(const MISTRING& countryname) {m_countryname = countryname;}

				void SetExtents(const DRECT2D& extents) {m_extents = extents;} 

				void SetLocation(const DPOINT2D& location) {m_location = location;}

				void SetName(const MISTRING& name) {m_name = name;}

			private:
				MISTRING m_name;			//!< Name of result
				MISTRING m_countryname;		//!< Name of result country
				DPOINT2D m_location;		//!< Location of result
				DRECT2D m_extents;			//!< Bounding box of result
				INT32 m_accuracy;			//!< Google Maps accuracy
			};

		enum SITE {
			SITE_Geonames,
			SITE_Google
			};

		void GetRecord (
			RECORD& record,
			int index
			) {record = m_list.GetIndexed(index);}

		//! Query first result and return as lat/lon point
		ERRVALUE QueryPoint (
			DPOINT2D& point,				//!< Point result
			SITE site,						//!< Site location to use
			MDLGPARENT dlgparent = NULL,	//!< Parent for status/progress dialog
			const MISTRING& dlgmsg = NULL	//!< Message to display in status dialog
			);

		//! Query results and add points to polyline
		ERRVALUE QueryPolyline (
			POLYLINE& polyline,				//!< Polyline result
			SITE site,						//!< Site location to use
			int MaxResults = 10,			//!< Number of results to use.  Max allowed results is 1000.
			MDLGPARENT dlgparent = NULL,	//!< Parent for status/progress dialog
			const MISTRING& dlgmsg = NULL	//!< Message to display in status dialog
			);

		//! Query results and return list of records
		ERRVALUE QueryRecordList (
			MILIST<RECORD>& List,			//!< List result
			SITE site,						//!< Site location to use
			int MaxResults = 10,			//!< Number of results to use.  Max allowed results is 1000.
			MDLGPARENT dlgparent = NULL,	//!< Parent for status/progress dialog
			const MISTRING& dlgmsg = NULL	//!< Message to display in status dialog
			);

		//! Set name of location to find
		void SetName (
			const MISTRING& name	//!< Name of location to find
			) {m_name = name;}

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_name;	// location name
		MILIST<RECORD> m_list;
		#endif	// GENERATING_DOXYGEN_OUTPUT
	
	};

}	// End namespace SOA

#endif //INC_SOA_GEONAMES_H
