/**
 * \file <spatmod/imagetargeters.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_ERS definition.
 *
 * \if NODOC
 * $Id: imagetargeters.h_v 1.1 2011/08/02 17:55:18 mju Exp $
 *
 * $Log: imagetargeters.h_v $
 * Revision 1.1  2011/08/02 17:55:18  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETERS_H
#define	INC_SPATMOD_IMAGETARGETERS_H

#ifndef	INC_SPATMOD_IMAGETARGETBIL_H
#include <spatmod/imagetargetbil.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Target for ER-Mapper ERS format.
class TARGET_ERS : public TARGET_BIL {
	public:

		//! Construct with specified input stage and path to target file.
		//! Target does not take ownership of input stage.
		TARGET_ERS (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const FILEPATH& filepath					//!< Path to target file
			);

		//! Destructor.
		virtual ~TARGET_ERS ();

		//! Set null value to output in header.
		//! Does not affect actual output values, use appropriate filter on source to ensure that invalid cells have specified null value.
		void SetNullValue (
			double NullValue
			) { m_NullValue = NullValue; m_HasNullValue = true; }


	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef TARGET_BIL BASECLASS;

		double m_NullValue;
		bool m_HasNullValue;

		// Unimplemented
		TARGET_ERS (const TARGET_ERS&);
		TARGET_ERS& operator= (const TARGET_ERS&);

		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================
}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETERS_H
